module datamod
  use modelvar
  use mpivar
  implicit none
  character(*), parameter :: dataroot=root//'data/'
  real(8) :: datmom(momdim),simmom(momdim),mcmom(momdim,momentmcdim),varcov(momdim,momdim),simcntr(momdim),datcntr(momdim)
contains
  subroutine readmom_mpi(filename)			!======= Reads the datmom.csv and mcmom.csv files ==============
    implicit none
    character(*), intent(in) :: filename
    integer :: c1,c2
    if(master) then
       open(unit=10, file=dataroot//'dat_'//filename//'.csv', status='old',readonly)
       read(10,*) (datmom(c1), c1=1,momdim)
       close(10)
       open(unit=10, file=dataroot//'mc_'//filename//'.csv', status='old', readonly)
       read(10,*) ((mcmom(c1,c2), c1=1,momdim), c2=1,momentmcdim)
       close(10)
    end if
    call MPI_BCAST(datmom,momdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
    call MPI_BCAST(mcmom,momdim*momentmcdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  end subroutine readmom_mpi

  subroutine genvarcov		!======= Subroutine calculates the inverse of the VARCOV matrix - output in varcov ============
    implicit none
    integer :: xx
    real(8) :: l1(momdim,1), l2(momdim,momdim),locavg(momdim),l5
    integer :: l4(momdim),c1
    xx=33
    l2=zero
    locavg=sum(mcmom,dim=2)/momentmcdim
    do c1=1,momentmcdim
       l1(:,1)=locavg-mcmom(:,c1)
       l2=l2+matmul(l1,transpose(l1))
    end do
    l2=l2/momentmcdim
    datavarcov=l2
    varcov=one
    call dlinrg(momdim,l2,momdim,varcov,momdim)			!Ouput of inv put in varcov

    call writeout_mpi
  contains
    subroutine writeout_mpi
      use mpivar
      implicit none
      if(master) then
         open(unit=10, file=root//outdir//'momstd.csv', status='replace')
         do c1=1,xx
            write(10,'(f20.10,",",f20.10)'),datmom(c1),sqrt(l2(c1,c1))
         end do
         l5=dble(0.0)
         do c1=xx-2,xx
            l5=l5+l2(c1,c1)
         end do
         l5=l5+2.0d0*(l2(xx,xx-1)+l2(xx,xx-2)+l2(xx-1,xx-2))
         write(10,'(f20.10,",",f20.10)'),1.0-sum(datmom(xx-2:xx)),sqrt(l5)
         do c1=xx+1,momdim
            write(10,'(f20.10,",",f20.10)'),datmom(c1),sqrt(l2(c1,c1))
         end do
         close(10)
      end if
    end subroutine writeout_mpi
  end subroutine genvarcov
end module datamod
