module estimate 
  use modelvar
  use datamod
  use mpivar
  use distr
  use moments
  character(3) :: ver
contains
  subroutine findpoint(guess,id,outp,outpval)
    implicit none
    real(8), intent(in) :: guess(estdim)
    character(*), intent(in) :: id
    real(8), intent(out) :: outp(estdim),outpval
    real(8) :: fval0, bestest(estdim)

    simrep=1000
    ver='3t'
    estmin=exp(varhuge)
    estrep=1
    !====================================================================================================================================
    !====== INSERT CALL TO PREFERRED OPTIMIZER HERE =====================================================================================
    !====== Minimize simdist (or simdist_f) using an optimizer that can handle the somewhat rough surface of the criterion function =====
    !====== use input in guess as the initial guess. Arg min in bestest and min in fval0. ===============================================
    !====================================================================================================================================

    if(master) then
       open(10,file=root//outdir//'point_'//id//'.csv',status='replace')
       write(10,'(f25.15)'),bestest
       write(10,'(f25.15)'),sigma
       close(10)
    end if
    outpval=fval0
    outp=bestest
  end subroutine findpoint

  subroutine bootstrap(length,name)
    implicit none
    integer, intent(in) :: length
    character(len=length), intent(in) :: name
    real(8) :: pointest(estdim),fval0,fval1,parest(estdim),oldparest(estdim)
    integer :: i,j
    !==== read in point estimate and set datcntr and simcntr =====
    call readpar_mpi(name,pointest) !pointest

    simrep=50 !bootstraps based on fewer simulation repetitions than point estimate
    ver='bt'
    call simdist(pointest,fval0)
    datcntr=datmom
    simcntr=simmom
    !=============================================================

    !======= bootstrap procedure ======
    do i=1,momentmcdim
       call MPI_BARRIER(MPI_COMM_WORLD,ierr)
       datmom=mcmom(:,i) !bootstrapped data moments are set
       estrep=i
       parest=pointest !let guess be given by pointest
       !====================================================================================================================================
       !====== INSERT CALL TO PREFERRED OPTIMIZER HERE =====================================================================================
       !====== Minimize simdist (or simdist_f) using an optimizer that can handle the somewhat rough surface of the criterion function =====
       !====== Let the arg min for the current version of datmom be contained in parest ====================================================
       !====================================================================================================================================

       if(master) then
          if(i == 1) then
             open(10,file=root//outdir//'bootres_'//name//'.csv',status='replace')
          else
             open(10,file=root//outdir//'bootres_'//name//'.csv',status='old',access='append')
          end if
          write(10,'(i5,", ",$)'),i
          do j=1,estdim-1
             write(10,'(f20.10,", ",$)'),parest(j)
          end do
          write(10,'(f20.10)'),parest(j)
          close(10)
       end if
    end do
    !===================================
  end subroutine bootstrap

  subroutine parset(inp) !take vector of estimated parameters and assign them to model parameters
    implicit none
    real(8), intent(in) :: inp(estdim)
    integer :: i,indx(en)
    c0=inp(1) !cost function parameter
    c1=inp(2) !cost function parameter
    eta=inp(3) !Aggregate entry rate
    Z=inp(4) !Average demand realization
    q_beta=inp(5) !common weibull shape parameter for all type dependent q-distributions
    z_gam=inp(6) !origin of weibull distribution for demand distribution
    z_beta=inp(7) !shape parameter for demand distribution
    errorsigmaY=inp(8) !Y measurement noise
    errorsigmaW=inp(9) !W measurement noise
    do i=1,en !type dependent q-distribution scale parameter
	sig(i)=inp(9+i)
        if(sig(i) .lt. 0.00001d0) sig(i)=zero
    end do
    do i=1,en-1 !phi distribution
	phi(i)=inp(9+en+i)/(one+sum(inp(9+en+1:8+2*en)))
    end do
    phi(en)=one/(one+sum(inp(9+en+1:8+2*en)))
    kappa=inp(estdim) !capital share
    Z_eta=(Z-Z_gam)/gammafct(one/Z_beta + one) !z_eta is the scale parameter
    call expectpi !determine the type dependent average profit realization (expectpi is in modelpar.f90)
    call avglabshare !determine labor 
    do i=1,en
       if(sig(i) .eq. zero) then
          elnq_sol(i)=zero
          eq(i)=one
       else
          elnq_sol(i)=elogq(sig(i),q_beta,one)
          eq(i)=one+sig(i)*gammafct(one/q_beta + one)
       end if
    end do
  end subroutine parset

  subroutine parset_sort(inp) !Same as parset, but sort the types in ascending order.
    implicit none
    real(8), intent(in) :: inp(estdim)
    integer :: i,indx(en)
    real(8) :: sig_loc(en),phi_loc(en)
    c0=inp(1)
    c1=inp(2)
    eta=inp(3)
    Z=inp(4)
    q_beta=inp(5)
    z_gam=inp(6)
    z_beta=inp(7)
    errorsigmaY=inp(8)
    errorsigmaW=inp(9)
    do i=1,en
	sig_loc(i)=inp(9+i)
        if(sig_loc(i) .lt. 0.00001d0) sig_loc(i)=zero
    end do
    do i=1,en-1
	phi_loc(i)=inp(9+en+i)/(one+sum(inp(9+en+1:8+2*en)))
    end do
    phi_loc(en)=one/(one+sum(inp(9+en+1:8+2*en)))
    kappa=inp(estdim)

    call sort_indx(sig_loc,indx)
    do i=1,en
	sig(i)=sig_loc(indx(i))
	phi(i)=phi_loc(indx(i))
    end do

    Z_eta=(Z-Z_gam)/gammafct(one/Z_beta + one)
    call expectpi
    call avglabshare
    do i=1,en
       if(sig(i) .eq. zero) then
          elnq_sol(i)=zero
          eq(i)=one
       else
          elnq_sol(i)=elogq(sig(i),q_beta,one)
          eq(i)=one+sig(i)*gammafct(one/q_beta + one)
       end if
    end do
  end subroutine parset_sort

  function simdist_f(inp)
    implicit none
    real(8), dimension(:), intent(in) :: inp
    real(8) :: simdist_f
    call simdist(bound(inp),simdist_f)
  end function simdist_f

  subroutine simdist(inp,outp) !Determine the distance between simulated auxiliary model and data auxiliary model.
    implicit none
    real(8), intent(in) :: inp(estdim)
    real(8), intent(out) :: outp
    integer :: err
    real(8) :: diff(momdim,1),reslt(1,1),res
    integer :: i,myrep,sim
    simcallcount=simcallcount+1

    call parset(inp) !sets structural parameters given input

    call findequil(err) !determine equilibrium values for given global model parameters
    if(err .eq. 1) then
       outp=exp(varhuge)
       RETURN
    end if

    if(master) then !master writes to screen
       write(6,'(" ")')
       write(6,'(a18,12a11,$)'),'','c0   ','c1   ','kappa  ','eta   ','g     ','q_beta  ','z_beta  ','z     ','z_gam   ','sigY   ', 'sigW   ','sigma  '
       do i=1,en
          write(6,'(a6,I1,a2,$)'),'   sig',i-1,'   '
       end do
       do i=1,en-1
          write(6,'(a6,I1,a2,$)'),'   phi',i-1,'   '
       end do
       write(6,'(a9,I1,a2)'),'      phi',en-1,'   '
       write(6,'(i4,2i7,12f11.4,$)'),estrep,simrep,simcallcount,c0,c1,kappa,eta,g,q_beta,z_beta,z,z_gam,errorsigmaY,errorsigmaW,sigma
       do i=1,en
          write(6,'(f9.4,$)'),sig(i)
       end do
       do i=1,en
          write(6,'(f9.4,$)'),phi(i)
       end do
    end if

    call avgsimmom(simmom) !Determine simulated auxiliary model.

    diff(:,1)=(simmom-simcntr)-(datmom-datcntr) !For regular estimation, simcntr and datcntr are zero. For bootstrap, moments are recentered.
    reslt=matmul(matmul(transpose(diff),varcov),diff)

    if(ISNAN(reslt(1,1))) then
       outp=exp(varhuge)
    else
       outp=reslt(1,1)
    end if

    if(master) write(6,'(f15.4)'),outp !master writes to screen
    if(outp .lt. estmin) then
	estmin=outp
        if(master) call writemomfit !master process writes best estimate to date.
    end if
    call MPI_BARRIER(MPI_COMM_WORLD,ierr) !everyone waits for all to be done.
  contains
    subroutine writemomfit
      implicit none
      integer :: regpnt(5)
      integer :: momorder(37)
      real(8) :: l1,l2,l3,welf,dc1,dc2,dc3,l_k0,l_k1,elnq_sol(en)
      integer :: ii
      momorder =(/ 19,1,3,8,2,4,9,10,11,36,12,14,20,13,15,21,22,23,37,5,6,7,16,17,18,24,25,26,27,29,30,28,34,35,31,32,33 /)
      l1=gammafct(one+one/q_beta)
      game=zero
      man_el=zero
      el=zero
      do ii=1,en
	 eq(ii)=one+sig(ii)*l1
         elnq_sol(ii)=elogq(sig(ii),q_beta,one)
         game=game+v(ii)*phi(ii)
      end do
      end_g=g_calc()
      call g_decomp(dc1,dc2,dc3)
      game=(game/(c0*(one+c1)*w))**(one/c1)
      em=eta/game
      call labdemand
      open(10,file=root//outdir//trim(ver)//'.txt')
      do ii=1,size(inp)
         write(10,'(f20.10)'),inp(ii)
      end do
      write(10,'("============================== Parameter Values ==========================")')
      write(10,'(a7," =",f15.5)'),'c0',c0
      write(10,'(a7," =",f15.5)'),'c1',c1
      write(10,'(a7," =",f15.5)'),'kappa',kappa
      write(10,'(a7," =",f15.5)'),'eta',eta
      write(10,'(a7," =",f15.5)'),'q_beta',q_beta
      write(10,'(a7," =",f15.5)'),'Z',z
      write(10,'(a7," =",f15.5)'),'z_beta',z_beta
      write(10,'(a7," =",f15.5)'),'z_gam',z_gam
      write(10,'(a7," =",f15.5)'),'g',g
      write(10,'(a7," =",f15.5)'),'sigY',errorsigmaY
      write(10,'(a7," =",f15.5)'),'sigW',errorsigmaW
      write(10,'(a7," =",f15.5)'),'sigma',sigma
      do i=1,en
         write(10,'(a3,I1,a3," =",f15.5)'),'sig',i-1,' ',sig(i)
      end do
      do i=1,en
         write(10,'(a3,I1,a3," =",f15.5)'),'phi',i-1,' ',phi(i)
      end do
      write(10,'(" ")')
      do i=1,en
         write(10,'(a1,I1,a5," =",f15.5)'),'v',i-1,' ',v(i)
      end do
      do i=1,en
         write(10,'(a3,I1,a3," =",f15.5)'),'Psi',i-1,' ',psi_v(i)
      end do
      do i=1,en
         write(10,'(a3,I1,a3," =",f15.5)'),'gam',i-1,' ',gam(i)
      end do
      write(10,'(a7," =",f15.5)'),'delta',delta
      write(10,'(a7," =",f15.5)'),'game',game
      do i=1,en
         write(10,'(a1,I1,a5," =",f15.5)'),'k',i-1,' ',k(i)
      end do
      do i=1,en
         write(10,'(a3,I1,a1,a2," =",f15.5)'),'E[q',i-1,']',' ',sig(i)*l1+one
      end do
      do i=1,en
         write(10,'(a2,I1,a4," =",f15.5)'),'pi',i-1,' ',pi(i)
      end do
      do i=1,en
         write(10,'(a5,I1,a1," =",f15.5)'),'E[lnq',i-1,']',elnq_sol(i)
      end do
      do i=1,en
         write(10,'(a5,I1,a1," =",f15.5)'),'E[q',i-1,']',eq(i)
      end do
      write(10,'(a7," =",f15.5)'),'end_g',end_g
      write(10,'(a7," =",f15.5)'),'- entry share',dc1/end_g
      write(10,'(a7," =",f15.5)'),'- residual share',dc2/end_g
      write(10,'(a7," =",f15.5)'),'- selection share',dc3/end_g
      write(10,'(a7," =",f15.5)'),'m',em
      write(10,'(a7," =",f15.5)'),'el',el
      write(10,'(a7," =",f15.5)'),'R&D share',(el-man_el)/el
      write(10,'(" ")')
      write(10,'("==============================    Moment fit   ==========================")')
      write(10,'("Distance = ",f25.10)'),outp
      write(10,'(" ")')
      write(10,'(a35,a15,a17)'),' ','     Data     ','  Simulation  '
      write(10,'("--------------------------------------------------------------")')

      do i=1,momdim
         write(10,'(a33,f15.5,f17.5)'),momnames(momorder(i)),datmom(momorder(i)),simmom(momorder(i))
      end do
      close(10)
    end subroutine writemomfit
  end subroutine simdist

  subroutine avgsimmom(simmom)
    implicit none
    real(8), intent(out) :: simmom(momdim)
    real(8) :: l1(momdim),mysimmom(momdim)
    integer :: i,ksmat(simrep)
    mysimmom=zero
    do i=1+myid,simrep,numprocs !Simulation repetitions split up between processes.
       call simul(ydat,wdat,ndat,seed+i*2) !Each simulation repetition has its own seed for simulation. Variable "seed" common to all processes.
       call momvect(l1,ydat,wdat,ndat)
       mysimmom=mysimmom+l1/dble(simrep)
    end do
    call MPI_ALLREDUCE(mysimmom,simmom,momdim,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr) !All processes send their own vector and all receive back the sum in simmom.
  end subroutine avgsimmom
end module estimate
