program main
  use mpivar
  use modelvar
  use datamod
  use distr
  use estimate
  implicit none
  real(8) :: parest(estdim),crit
  integer :: i,j
  character(*), parameter :: name='priv' !this sets the name of a number of output files. This is also used to then read these output files in other parts of the program.

  call mpi_ini
  call mcini_mpi
  kill_q_var=.FALSE.
  kill_a_var=.FALSE.
  kill_g=.FALSE.

  N=4872	!Entire private sector
!!$ N=2051	!sector 2 Manufacturing
!!$ N=651	!sector 4 Construction
!!$ N=1584	!sector 5 Wholesale and Retail

  w=190.23911301 !all private
!!$ w=190.65978038 !2
!!$ w=191.84870108 !4
!!$ w=187.71999745 !5

  T=6
  r=0.05
  per=26

  call readmom_mpi('mom')
  call genvarcov

  allocate(ydat(N,T),wdat(N,T),ndat(N,T))

  datcntr=zero
  simcntr=zero

!!$  call findpoint(parest,name,parest,crit) !Modify to include optimizer before call. Provide guess in parest prior to call.
!!$  call bootres(len(name),name) !Modify to include optimizer before call
  call readpar_mpi('priv',parest)
  call table_V !Produces data for table V
  call table_VI !Produces data for table VI
  call sim_moments !Produces simulated moments for estimated model and counterfactuals
  call fiteval(len(name),name)
  call pareval(len(name),name)

  call mpi_term
end program main

subroutine sim_moments
  use mpivar
  use modelvar
  use estimate
  implicit none
  real(8) :: crit,parest(estdim)

  call readpar_mpi('priv',parest)
  if(master) open(11,file=root//outdir//'sim_moments_data.txt',status='replace')

  simrep=1000
  ver='priv_fit'
  estmin=zero !stops any output
  call simdist(parest,crit)
  if(master) then
     write(11,'(a50)'),"Estimated model fit                "
     call momwrite
     write(11,'(" ")')
  end if

  kill_g=.TRUE.
  call simdist(parest,crit)
  if(master) then
     write(11,'(a50)'),"Estimated model fit (constant price index)"
     call momwrite
     write(11,'(" ")')
  end if
  kill_g=.FALSE.

  parest(8)=zero !Y measurement error
  parest(9)=zero !W measurement error
  call simdist(parest,crit)
  if(master) then
     write(11,'(a50)'),"Estimated model fit (zero measurement error) "
     call momwrite
     write(11,'(" ")')
  end if

  parest(6)=parest(4) !sets demand realization variance to zero
  call simdist(parest,crit)
  if(master) then
     write(11,'(a50)'),"Estimated model fit (zero measurement error and demand variance) "
     call momwrite
     close(11)
  end if
contains
  subroutine momwrite
    implicit none
    integer, dimension(37), parameter :: momorder =(/ 19,1,3,8,2,4,9,10,11,36,12,14,20,13,15,21,22,23,37,5,6,7,16,17,18,24,25,26,27,29,30,28,34,35,31,32,33 /) 
    integer :: i
    write(11,'(70("-",$))')
    write(11,'(" ")')
    write(11,'(a7," =",f15.4)'),'c0',c0
    write(11,'(a7," =",f15.4)'),'c1',c1
    write(11,'(a7," =",f15.4)'),'kappa',kappa
    write(11,'(a7," =",f15.4)'),'eta',eta
    write(11,'(a7," =",f15.4)'),'q_beta',q_beta
    write(11,'(a7," =",f15.4)'),'Z',z
    write(11,'(a7," =",f15.4)'),'z_beta',z_beta
    write(11,'(a7," =",f15.4)'),'z_gam',z_gam
    write(11,'(a7," =",f15.4)'),'g',g
    write(11,'(a7," =",f15.4)'),'sigY',errorsigmaY
    write(11,'(a7," =",f15.4)'),'sigW',errorsigmaW
    write(11,'(a7," =",f15.4)'),'sigma',sigma
    do i=1,en
       write(11,'(a3,I1,a3," =",f15.4)'),'sig',i-1,' ',sig(i)
    end do
    do i=1,en
       write(11,'(a3,I1,a3," =",f15.4)'),'phi',i-1,' ',phi(i)
    end do
    write(11,'(" ")')
    do i=1,en
       write(11,'(a1,I1,a5," =",f15.4)'),'v',i-1,' ',v(i)
    end do
    do i=1,en
       write(11,'(a3,I1,a3," =",f15.4)'),'Psi',i-1,' ',psi_v(i)
    end do
    do i=1,en
       write(11,'(a3,I1,a3," =",f15.4)'),'gam',i-1,' ',gam(i)
    end do
    write(11,'(a7," =",f15.4)'),'delta',delta
    
    write(11,'("Distance = ",f25.10)'),crit
    write(11,'(" ")')
    write(11,'(a35,a15,a17)'),' ','     Data     ','  Simulation  '
    write(11,'(70("-",$))')
    write(11,'(" ")')

    do i=1,momdim
       write(11,'(a33,f15.5,f17.5)'),momnames(momorder(i)),datmom(momorder(i)),simmom(momorder(i))
    end do
  end subroutine momwrite
end subroutine sim_moments

subroutine table_V
  use mpivar
  use modelvar
  implicit none
  real(8) :: fhk(7),parest(estdim)

  call readpar_mpi('priv',parest)

  if(master) open(11,file=root//outdir//'table_V_data.txt',status='replace')
  call fullsim(parest,fhk)
  if(master) then
     write(11,'(a50)'),"Estimated model with entry            "
     call fhkwrite
     write(11,'(" ")')
  end if

  parest(8)=zero !Y measurement error
  parest(9)=zero !W measurement error
  call fullsim(parest,fhk)
  if(master) then
     write(11,'(a50)'),"Counterfactual 1                 "
     call fhkwrite
     write(11,'(" ")')
  end if

  parest(7)=parest(4) !sets demand realization variance to zero
  call fullsim(parest,fhk)
  if(master) then
     write(11,'(a50)'),"Counterfactual 2                 "
     call fhkwrite
     write(11,'(" ")')
  end if

  kill_q_var=.TRUE. !sets q-variation to zero in simulations
  kill_a_var=.TRUE. !sets age-variation to zero in simulations
  call fullsim(parest,fhk)
  if(master) then
     write(11,'(a50)'),"Counterfactual 3                 "
     call fhkwrite
     close(11)
  end if
  kill_q_var=.FALSE. 
  kill_a_var=.FALSE.
contains
  subroutine fhkwrite
    implicit none
    write(11,'(50("-",$))')
    write(11,'(" ")')
    write(11,'(a25,f20.10)'),"Within",fhk(1)
    write(11,'(a25,f20.10)'),"Between",fhk(2)
    write(11,'(a25,f20.10)'),"Cross",fhk(3)
    write(11,'(a25,f20.10)'),"Exit",fhk(5)
    write(11,'(a25,f20.10)'),"Entry",fhk(4)
    write(11,'(a25,f20.10)'),"Prod Growth (survivor)",fhk(7)
  end subroutine fhkwrite
end subroutine table_V

subroutine table_VI
  use mpivar
  use modelvar
  implicit none
  real(8) :: res(12+en*11),parest(estdim)

  if(master) open(11,file=root//outdir//'table_VI_data.txt',status='replace')
  call readpar_mpi('priv',parest)
  call esteval(parest,res) !runs parset and findequil
  if(master) then
     write(11,'(a50)'),"Estimated Model (sigma=1.00)           "
     call gwrite
     write(11,'(" ")')
  end if

  call readpar_mpi('priv_sig050',parest)
  call esteval(parest,res) !runs parset and findequil
  if(master) then
     write(11,'(a50)'),"Estimated Model (sigma=0.50)           "
     call gwrite
     write(11,'(" ")')
  end if

  call readpar_mpi('priv_sig075',parest)
  call esteval(parest,res) !runs parset and findequil
  if(master) then
     write(11,'(a50)'),"Estimated Model (sigma=0.75)           "
     call gwrite
     write(11,'(" ")')
  end if

  call readpar_mpi('priv_sig125',parest)
  call esteval(parest,res) !runs parset and findequil
  if(master) then
     write(11,'(a50)'),"Estimated Model (sigma=1.25)           "
     call gwrite
     close(11)
  end if
contains
  subroutine gwrite
    write(11,'(50("-",$))')
    write(11,'(" ")')
    write(11,'(a25,f20.10)'),"Growth Rate",res(1)
    write(11,'(a25,f20.10)'),"Entry/Exit",res(2)
    write(11,'(a25,f20.10)'),"Redidual",res(3)
    write(11,'(a25,f20.10)'),"Selection",res(4)
  end subroutine gwrite
end subroutine table_VI

subroutine readpar_mpi(name,outp)
  use mpivar
  use modelvar
  implicit none
  character(*), intent(in) :: name
  real(8), intent(out) :: outp(estdim)
  real(8) :: parest(estdim)
  integer :: i
  if(master) then
     open(10,file=root//outdir//"point_"//name//".csv",status='old')
     read(10,*) (parest(i), i=1,estdim)
     read(10,*) sigma
     forall(i=1:estdim,parest(i) .le. lo(i)) parest(i)=lo(i)+1.0e-10 !fix potential problem that written output may have been rounded down to lower bound
     close(10)
  end if
  call MPI_BCAST(parest,estdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  call MPI_BCAST(sigma,1,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)
  outp=parest
end subroutine readpar_mpi

subroutine fullsim(parest,outp)
  use mpivar
  use modelvar
  use datamod
  use estimate
  use moments
  implicit none
  real(8), intent(in) :: parest(estdim)
  real(8), intent(out) :: outp(7)
  integer, parameter :: pardim=estdim+15+13*en
  real(8) :: parout(pardim-estdim),err,surv(T),decomp(6),decomp1(6),tot,distr(2,T),distr1(2,T),tot1,agg_growth,agg_1
  real(4), allocatable :: yy(:,:),ww(:,:),nn(:,:)
  integer :: i,N_e,j,ii,rep
  rep=100000 !rep sets the number of simulation repetitions.
  call esteval(parest,parout) !runs parset and findequil

  surv=zero
  decomp=zero
  distr=zero
  tot1=zero
  agg_1=zero
  do ii=myid+1,rep,numprocs
     call entry_simul(seed+ii)
     N_e=size(yy_e(:,1))
     if(allocated(yy)) deallocate(yy,ww,nn)
     allocate(yy(N+N_e,T),ww(N+N_e,T),nn(N+N_e,T))
     call simul(yy(1:N,:),ww(1:N,:),nn(1:N,:),seed+ii*19)
     yy(N+1:N+N_e,:)=yy_e
     ww(N+1:N+N_e,:)=ww_e
     nn(N+1:N+N_e,:)=nn_e
     agg_1=agg_1+(sum(dble(yy(1:N,T)))*sum(dble(nn(1:N,1)))/(sum(dble(yy(1:N,1)))*sum(dble(nn(1:N,T)))) )**(one/dble(T))-one

     call FHK(decomp1)
     tot=sum(decomp1(1:4))-decomp1(5)
     forall(i=1:4) decomp(i)=decomp(i)+decomp1(i)/tot
     decomp(5)=decomp(5)-decomp1(5)/tot
     decomp(6)=decomp(6)+decomp1(6)
     tot1=tot1+tot
  end do
  decomp1=decomp/dble(rep)
  agg_1=agg_1/dble(rep)
  call MPI_ALLREDUCE(decomp1,decomp,6,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)
  call MPI_ALLREDUCE(agg_1,agg_growth,1,MPI_DOUBLE_PRECISION,MPI_SUM,MPI_COMM_WORLD,ierr)

  outp(1:6)=decomp
  outp(7)=agg_growth
CONTAINS
  subroutine mom(outp)
    implicit none
    real(8), intent(out) :: outp(2,T)
    do i=1,T
       outp(1,i)=avg(yy(:,i)/nn(:,i))
       outp(2,i)=sqrt(variance(yy(:,i)/nn(:,i)))
    end do
  end subroutine mom

  subroutine FHK(outp)
    implicit none
    real(8), intent(out) :: outp(6)
    integer :: TN
    real(8) :: shares(N+N_e,2),prod(N+N_e,2),aggprod(2),totN(2)
    TN=N+N_e
    totN(1)=sum(nn(:,1))
    totN(2)=sum(nn(:,T))
    do i=1,TN
       shares(i,1)=nn(i,1)/totN(1)
       shares(i,2)=nn(i,T)/totN(2)
       if(shares(i,1) > zero) prod(i,1)=yy(i,1)/nn(i,1)
       if(shares(i,2) > zero) prod(i,2)=yy(i,T)/nn(i,T)
    end do
    aggprod(1)=sum(shares(:,1)*prod(:,1))
    aggprod(2)=sum(shares(:,2)*prod(:,2))
    outp=zero
    outp(6)=(aggprod(2)/aggprod(1))**(one/dble(T))-one
    do i=1,TN
       if(shares(i,1) > zero .and. shares(i,2) > zero) then
          outp(1)=outp(1)+shares(i,1)*(prod(i,2)-prod(i,1))
          outp(2)=outp(2)+(shares(i,2)-shares(i,1))*(prod(i,1)-aggprod(1))
          outp(3)=outp(3)+(shares(i,2)-shares(i,1))*(prod(i,2)-prod(i,1))
       else if(shares(i,1) > zero .and. shares(i,2) == zero) then
          outp(5)=outp(5)+shares(i,1)*(prod(i,1)-aggprod(1))
       else if(shares(i,1) == zero .and. shares(i,2) > zero) then
          outp(4)=outp(4)+shares(i,2)*(prod(i,2)-aggprod(1))
       end if
    end do
  end subroutine FHK
end subroutine fullsim

subroutine esteval(inp,outp)
  use modelvar
  use estimate
  implicit none
  real(8), intent(in) :: inp(estdim)
  real(8), intent(out) :: outp(12+en*11)
  real(8) :: l1,l2,l3,welf,dc1,dc2,dc3,l_k0,l_k1,err
  integer :: ii,jj
  call parset_sort(inp)
  call findequil(err)

  l1=gammafct(one+one/q_beta)
  game=zero
  man_el=zero
  el=zero
  end_g=zero
  dc1=zero
  dc2=zero
  dc3=zero
  do ii=1,en
     game=game+v(ii)*phi(ii)
  end do
  end_g=g_calc()
  call g_decomp(dc1,dc2,dc3)
  game=(game/(c0*(one+c1)*w))**(one/c1)
  em=eta/game
  call labdemand

  outp(1)=end_g !implied growth
  outp(2)=dc1/end_g !entry/exit share of growth
  outp(3)=dc2/end_g !residual share of growth
  outp(4)=dc3/end_g !selection share of growth
  outp(5)=em !mass of potential entrants
  outp(6)=el !labor supply
  outp(7)=(el-man_el)/el !share of labor force engaged in innovation
  outp(8)=delta
  outp(9)=game
  do ii=1,en
     outp(10+(ii-1)*11)=pi(ii)
     outp(11+(ii-1)*11)=v(ii)
     outp(12+(ii-1)*11)=psi_v(ii)
     outp(13+(ii-1)*11)=sig(ii)*(log(2.0d0)**(one/q_beta))+one !median q
     outp(14+(ii-1)*11)=sig(ii)*l1+one !average q
     outp(15+(ii-1)*11)=elnq_sol(ii) !average log q
     outp(16+(ii-1)*11)=gam(ii)
     outp(17+(ii-1)*11)=k(ii)
     outp(18+(ii-1)*11)=phi(ii)
     outp(19+(ii-1)*11)=(gam(ii)/(delta-gam(ii)))/log(delta/(delta-gam(ii)))
     if(isnan(outp(19+(ii-1)*11))) outp(19+(ii-1)*11)=one
     outp(20+(ii-1)*11)=capem(ii)/sum(capem)
  end do
  jj=20+(en-1)*11
  outp(jj+1)=dc1
  outp(jj+2)=dc2
  outp(jj+3)=dc3
end subroutine esteval

subroutine mcini_mpi
  use modelvar
  use mpivar
  implicit none
  call rnopt(5) !Can easily find better RNG than IMSL's MCG. Use of KISS() instead gave same results.
  seed=234567891
end subroutine mcini_mpi
