module modelvar !Module holds global model parameters and a number of helper routines.
  use Numerical_Libraries
  use distr
  implicit none
  integer, parameter :: estdim=15
  character(*), parameter :: root='/home/', outdir='output/' !set root directory before running.
  integer, parameter :: momentmcdim=500
  integer(4), parameter :: kbar=500,momdim=37,&
       &kdim=2*kbar*(kbar+2),&
       &seedmult=int(2.0e9),&
       &en=3 !the number of types
  integer(4) :: N,T,seed,estrep,simrep,simcallcount,per,vertflag
  real(16), parameter :: pid32=3.1415926535897932384626433832795
  real(8), allocatable :: yy_e(:,:),ww_e(:,:),nn_e(:,:)
  real(8) :: x1,x2,x3,datavarcov(momdim,momdim)
  logical :: kill_q_var, kill_a_var,kill_g
  real(8), parameter :: &
       &one=1.0d0,&
       &zero=0.0d0,&
       &vartiny=tiny(vartiny),&
       &varhuge=log(huge(varhuge))-1.0e-12,&
       &expvarhuge=huge(expvarhuge)
  real(4), allocatable :: ydat(:,:),wdat(:,:),ndat(:,:)
  real(8) :: sizedist(kdim),sizecdf(kbar,en), estmin, &
       &kappa,&
       &r,&
       &delta,&
       &w,&
       &c0,&
       &c1,&
       &tau,&
       &p,&
       &k(en),& !distribution of product mass over types.
       &pi(en),& !type conditional profit expectation
       &Z,&
       &phi(en),& !ex ante type pdf
       &eta,&
       &gam(en),& !type conditional creation rate choice
       &game,& !entrant creation rate choice
       &psi_v(en),& !type conditional value of innovation
       &v(en),& !type conditional value of product
       &g,&
       &ghat,&
       &z_beta,&
       &z_eta,&
       &z_gam,&
       &sig(en),& !type conditional q-realization spread
       &elnq_sol(en),&
       &eq(en),&
       &labshare(en),&
       &capshare(en),&
       &q_beta,&
       &errorsigmaY,&
       &errorsigmaW,&
       &em,& !mass of potential entrants
       &el,&
       &man_el,&
       &end_g,&
       &sigma,&
       &ztilde,&
       &capem(en) !steady state firm mass distribution over types.
!                                                 c0      c1       eta         z    q-beta  z_gam     z_beta,  errY,  errW,      sig1,           sig2,          sig3,    phi1,   phi2     kappa     sigma
  real(8), dimension(estdim), parameter :: lo=(/    0.05d0,0.10d0, 0.005d0,    10.0d0, 0.1d0,    10.0d0, 0.1d0,  zero,  zero,         zero,         zero,         zero,    zero,    zero,    1.0d0 /) !lo
  real(8), dimension(estdim), parameter :: hi=(/ 5000.00d0,10.0d0,   0.4d0, 50000.0d0, 5.0d0, 20000.0d0, 5.0d0,   one,   one,        5.0d0,        5.0d0,        5.0d0, 100.0d0, 100.0d0, 1000.0d0 /) !hi
  character(*), dimension(37), parameter :: momnames=(/ &
       &'E[Y]_1',& !1
       &'E[W]_1',& !2
       &'Std[Y]_1',& !3
       &'Std[W]_1',& !4
       &'Corr[Y,W]_1',& !5
       &'Corr[Y/N,N]_1',& !6
       &'Corr[Y/N,Y]_1',& !7
       &'Med[Y]_1',& !8
       &'Med[W]_1',& !9
       &'E[Y/N]_1',& !10
       &'Std[Y/N]_1',& !11
       &'E[Y]_T',& !12
       &'E[W]_T',& !13
       &'Std[Y]_T',& !14
       &'Std[W]_T',& !15
       &'Corr[Y,W]_T',& !16
       &'Corr[Y/N,N]_T',& !17
       &'Corr[Y/N,Y]_T',& !18
       &'Survivors_T',& !19
       &'Med[Y]_T',& !20
       &'Med[W]_T',& !21
       &'E[Y/N]_T',& !22
       &'Std[Y/N]_T',& !23
       &'Corr[(Y/N)_1,(Y/N)_2]',& !24
       &'Corr[(Y/N)_T-1,(Y/N)_T]',& !25       
       &'Corr[(Y/N)_2-(Y/N)_1,(Y/N)_1]',& !26
       &'Corr[(Y/N)_T-(Y/N)_T-1,(Y/N)_T-1]',& !27
       &'Corr[(Y_2-Y_1)/Y_1,Y_1]',& !28
       &'E[(Y_2-Y_1)/Y_1]',& !29
       &'Std[(Y_2-Y_1)/Y_1]',& !30
       &'Within',& !31
       &'Between',& !32
       &'Cross',& !33
       &'Corr[(Y_2-Y_1)/Y_1,Y_1/N_1]',& !34
       &'Corr[(N_2-N_1)/N_1,Y_1/N_1]',& !35
       &'Med[Y_1/N_1]',& !36
       &'Med[Y_T/N_T]' /) !37

  interface bound
     module procedure bound_d, bound_s
  end interface
  interface inv_bound
     module procedure inv_bound_d, inv_bound_s
  end interface
  interface ergodist
     module procedure ergodist_a
  end interface
contains
  function g_calc() !Determine final consumption good growth rate. Take globals as input.
    implicit none
    real(8) :: g_calc,loc_delta,E_tau(en),base
    integer :: i
    g_calc=zero
    if(sigma .eq. one) then
       do i=1,en
          g_calc=g_calc+(eta*phi(i)+k(i)*gam(i))*elogq(sig(i),q_beta,one)
       end do
    end if
    if(sigma < one) then
       do i=1,en
          g_calc=g_calc+(eta*phi(i)+k(i)*gam(i))*e_tau_int_3(sig(i),q_beta,one)
       end do
    end if
    if(sigma > one) then
       base=zero
       do i=1,en
          base=base+k(i)*e_tau_int_1(sig(i),q_beta,one)
       end do
       do i=1,en
          e_tau(i)=base*e_tau_int_2(sig(i),q_beta,one)
       end do
       do i=1,en
          g_calc=g_calc+(eta*phi(i)+k(i)*gam(i))*(e_tau(i)-one)/(sigma-one)
       end do
    end if
  end function g_calc

  subroutine g_decomp(entr,resid,selec) !determine growth decomposition. Take globals as input.
    implicit none
    real(8), intent(out) :: entr, resid, selec
    integer :: i
    real(8) :: loc1,int1(en),int2(en),base
    entr=zero
    resid=zero
    selec=zero
    if(sigma <= one) then
       do i=1,en
          loc1=f1_priv(sig(i),q_beta)
          entr=entr+eta*phi(i)*loc1
          resid=resid+gam(i)*phi(i)*loc1
          selec=selec+(k(i)-phi(i))*gam(i)*loc1
       end do
    else
       do i=1,en
          int1(i)=e_tau_int_1(sig(i),q_beta,one)
          int2(i)=e_tau_int_2(sig(i),q_beta,one)
       end do       

       base=zero
       do i=1,en
        base=base+phi(i)*int1(i)
       end do
       do i=1,en
          entr=entr+eta*phi(i)*(int2(i)*base-one)/(sigma-one)
       end do

       base=zero
       loc1=zero
       do i=1,en
          base=base+phi(i)*gam(i)*int1(i)
          loc1=loc1+phi(i)*gam(i)
       end do
       base=base/loc1
       do i=1,en
          resid=resid+phi(i)*gam(i)*(int2(i)*base-one)/(sigma-one)
       end do
       selec=g_calc()-entr-resid
    end if
    if(real(entr+resid+selec) .ne. real(g_calc())) then
       print *,g_calc(),entr+resid+selec
       stop'fatal: decomposition does not add to total growth.'
    end if
  contains
    double precision function f1_priv(inp1,inp2)
      implicit none
      real(8), intent(in) :: inp1, inp2
      if(sigma .eq. one) f1_priv=elogq(inp1,inp2,one)
      if(sigma .lt. one) f1_priv=e_tau_int_3(inp1,inp2,one)
    end function f1_priv
  end subroutine g_decomp

  double precision function e_tau_int_1(inp_sig,inp_beta,inp_gam)
    implicit none
    real(8), intent(in) :: inp_sig, inp_beta, inp_gam
    real(8) :: errest
    if(inp_sig .eq. zero) then
       e_tau_int_1=one
       return
    end if
    call erset(0,0,-1)
    call dqdagi(int_priv,inp_gam,1,dble(1.0e-7),dble(1.0e-7),e_tau_int_1,errest)
    call erset(0,2,2)
  contains
    double precision function int_priv(inp)
      implicit none
      real(8), intent(in) :: inp
      int_priv=weibull_pdf(inp-inp_gam,inp_beta,inp_sig)*(markup(inp)**(sigma-one))
    end function int_priv
  end function e_tau_int_1

  double precision function e_tau_int_2(inp_sig,inp_beta,inp_gam)
    implicit none
    real(8), intent(in) :: inp_sig, inp_beta, inp_gam
    real(8) :: errest
    if(inp_sig .eq. zero) then
       e_tau_int_2=one
       return
    end if
    call erset(0,0,-1)
    call dqdagi(int_priv,inp_gam,1,dble(1.0e-7),dble(1.0e-7),e_tau_int_2,errest)
    call erset(0,2,2)
  contains
    double precision function int_priv(inp)
      implicit none
      real(8), intent(in) :: inp
      int_priv=weibull_pdf(inp-inp_gam,inp_beta,inp_sig)*((inp/markup(inp))**(sigma-one))
    end function int_priv
  end function e_tau_int_2

  double precision function e_tau_int_3(inp_sig,inp_beta,inp_gam)
    implicit none
    real(8), intent(in) :: inp_sig, inp_beta, inp_gam
    real(8) :: errest
    if(inp_sig .eq. zero) then
       e_tau_int_3=zero
       return
    end if
    call erset(0,0,0)
    call dqdagi(int_priv,inp_gam,1,dble(1.0e-8),dble(1.0e-8),e_tau_int_3,errest)
    if(n1rty(1) > 3) e_tau_int_3=zero
    call erset(0,2,2)
  contains
    double precision function int_priv(inp)
      implicit none
      real(8), intent(in) :: inp
      int_priv=weibull_pdf(inp-inp_gam,inp_beta,inp_sig)*(inp**(sigma-one)-one)/(sigma-one)
    end function int_priv
  end function e_tau_int_3

  subroutine ergodist_a(inp,outp)
    implicit none
    real(8), dimension(:), intent(in) :: inp
    integer, intent(out) :: outp(size(inp),2)
    real(8) :: loc_msum,x,xx
    integer :: i,j,jj
    loc_msum=sum(capem)
    do i=1,size(inp)
       x=capem(1)/loc_msum
       j=1
       do while(inp(i) .gt. x)
          j=j+1
          x=x+capem(j)/loc_msum
       end do
       outp(i,1)=j
       xx=(inp(i)-(x-capem(j)/loc_msum))/(capem(j)/loc_msum)
       if(xx .gt. one) pause 'xx greater than one.'
       jj=1
       do while(xx .gt. sizecdf(jj,j))
          jj=jj+1
       end do
       outp(i,2)=jj
    end do
  end subroutine ergodist_a

  double precision function c(inp)
    implicit none
    real(8), intent(in) :: inp
    c=c0*(inp**(1.0d0+c1))
  end function c

  double precision function cprime(inp)
    implicit none
    real(8), intent(in) :: inp
    cprime=(1.0d0+c1)*c0*(inp**c1)
  end function cprime

  double precision function markup(inp) !provides the markup determined by the combination of monopoly pricing and limit pricing
    implicit none
    real(8), intent(in) :: inp
    if(sigma .le. one) then
       markup=inp
    else
       markup=min(inp,sigma/(sigma-one))
    end if
  end function markup

  double precision function pifct(inp)
    implicit none
    real(8), intent(in) :: inp
    pifct=one-one/markup(inp)
  end function pifct

  subroutine avglabshare
    implicit none
    real(8) :: reslt, errest,loc_sig
    integer :: i
    do i=1,en
       labshare(i)=w*(one-pi(i))/(w+kappa)
       capshare(i)=kappa*(one-pi(i))/(w+kappa)
    end do
  contains
    double precision function qint(inp)
      implicit none
      real(8), intent(in) :: inp
      qint=(one-weibull_cdf(inp-one,q_beta,loc_sig))/(inp**2)
    end function qint
  end subroutine avglabshare    

  subroutine labdemand	!expectpi must be called prior to labdemand call
    implicit none
    real(8) :: ell_i,ell_tau(en),loc_sig,reslt,errest
    integer :: i
    ell_i=zero
    do i=1,en
       ell_i=ell_i+k(i)*c(gam(i))*Z
    end do
    ell_i=ell_i+em*c(eta/em)*Z

    do i=1,en
       ell_tau(i)=k(i)*delta*Z*(one-pi(i))/( (w+kappa)*( delta-g*(one-sigma) ) )
    end do
    man_el=sum(ell_tau)
    el=man_el+ell_i
  contains
    double precision function mint(inp)
      implicit none
      real(8), intent(in) :: inp
      mint=(inp**(-one-sigma))*(one-weibull_cdf(inp-one,q_beta,loc_sig))
    end function mint
  end subroutine labdemand

  subroutine expectpi !calculate type dependent expected profit realization
    implicit none
    real(8) :: reslt, errest,loc_sig
    integer :: i
    do i=1,en
       if(sig(i) .eq. zero) then
          pi(i)=zero
          cycle
       end if
       loc_sig=sig(i)
       call erset(0,0,0)
       if(sigma .le. one) then
          call dqdagi(qint,one,1,dble(1.0e-7),dble(1.0e-7),pi(i),errest)
       else
          call dqdag(qint,one,sigma/(sigma-one),dble(1.0e-8),dble(1.0e-8),2,pi(i),errest)
       end if
       if(n1rty(1) > 3) pi(i)=pifct(one)
       call erset(0,2,2)
    end do
  contains
    double precision function qint(inp)
      implicit none
      real(8), intent(in) :: inp
      qint=(inp**(-2.0d0))*(one-weibull_cdf(inp-one,q_beta,loc_sig))
    end function qint
  end subroutine expectpi

  double precision function elogq(inp_sig,inp_beta,inp_gam)
    implicit none
    real(8), intent(in) :: inp_sig, inp_beta, inp_gam
    real(8) :: errest
    if(inp_sig .eq. zero) then
       elogq=zero
       return
    end if
    call erset(0,0,0)
    call dqdagi(int_priv,inp_gam,1,dble(1.0e-7),dble(1.0e-7),elogq,errest)
    if(n1rty(1) > 3) elogq=zero
    call erset(0,2,2)
  contains
    double precision function int_priv(inp)
      implicit none
      real(8), intent(in) :: inp
      int_priv=weibull_pdf(inp-inp_gam,inp_beta,inp_sig)*log(inp)
    end function int_priv
  end function elogq

  double precision function gamfonc(x)
    implicit none
    real(8), intent(in) :: x
    gamfonc=(x/(c0*(one+c1)*w))**(one/c1)
  end function gamfonc

  double precision function dist(inpa,inpb)
    implicit none
    real(8), dimension(:), intent(in) :: inpa, inpb
    dist=maxval(abs(inpa-inpb))
  end function dist

  function bound_d(inp)
    implicit none
    real(8), intent(in) :: inp(estdim)
    real(8) :: bound_d(estdim)
    integer :: i
    bound_d=lo+(hi-lo)/(one+exp(min(varhuge,inp)))
    do i=10,12
    	if(isnan(bound_d(i))) bound_d(i)=lo(i)+vartiny
    end do
  end function bound_d

  function inv_bound_d(inp)
    implicit none
    real(8) :: inv_bound_d(estdim)
    real(8), intent(in) :: inp(estdim)
    inv_bound_d=log((hi+vartiny-inp)/(inp-lo+vartiny))
  end function inv_bound_d

  function bound_s(inp)
    implicit none
    real(4), intent(in) :: inp(estdim)
    real(8) :: bound_s(estdim)
    bound_s=lo+(hi-lo)/(one+exp(min(varhuge,inp)))
  end function bound_s

  function inv_bound_s(inp)
    implicit none
    real(8) :: inv_bound_s(estdim)
    real(4), intent(in) :: inp(estdim)
    inv_bound_s=log((hi-inp)/(inp-lo))
  end function inv_bound_s

  subroutine momvect(outp,inpy,inpw,inpn)		!This subroutine selects a set of moments from the panel and reports them in an Mx1 vector.
    use moments
    use kernel
    implicit none
    real(4), intent(in) :: inpy(N,T),inpw(N,T),inpn(N,T)
    real(8), intent(out) :: outp(momdim)
    integer :: N1,T1
    real(8) :: l2,l3,l4,l5,l6,ssxx,ssyy,ssxy,l7,l8,loutp,l9,l10,avgprod(2),decomp(4),krnlout(5,2),krnlinp_reg(N,2),krnlinp_pdf(N,1)
    real(8) :: locshare(size(inpy,dim=1),2),locprod(size(inpy,dim=1),2),deltashare(size(inpy,dim=1)),deltaprod(size(inpy,dim=1))
    real(4) :: l11(size(inpy,dim=1)),l12(size(inpy,dim=1))
    integer :: i,j,c2,c3
    N1=size(inpy,dim=1)
    T1=size(inpy,dim=2)
    
    l11=inpy(:,1)/inpn(:,1)
    outp(1)=avg(inpy(:,1),real(0))
    outp(2)=avg(inpw(:,1),real(0))
    outp(3)=sqrt( variance(inpy(:,1),real(outp(1)),real(0)) )
    outp(4)=sqrt( variance(inpw(:,1),real(outp(2)),real(0)) )
    outp(5)=corr(inpy(:,1),inpw(:,1),real(0))
    outp(6)=corr(l11,inpn(:,1),real(0))
    outp(7)=corr(l11,inpy(:,1),real(0))
    outp(8)=medianval(inpy(:,1),real(0)) !Y Median
    outp(9)=medianval(inpw(:,1),real(0)) !W median
    outp(10)=avg(l11,real(0)) !Y/N mean
    outp(11)=sqrt( variance(l11,real(outp(10)),real(0)) ) !Y/N std
    outp(36)=medianval(l11,real(0)) !NEW

    l11=inpy(:,T1)/inpn(:,T1)
    outp(12)=avg(inpy(:,T1),real(0))
    outp(13)=avg(inpw(:,T1),real(0))
    outp(14)=sqrt( variance(inpy(:,T1),real(outp(12)),real(0)) )
    outp(15)=sqrt( variance(inpw(:,T1),real(outp(13)),real(0)) )
    outp(16)=corr(inpy(:,T1),inpw(:,T1),real(0))
    outp(17)=corr(l11,inpn(:,T1),real(0))
    outp(18)=corr(l11,inpy(:,T1),real(0))
    c3=0
    do i=1,N1
       if(inpy(i,T1) .eq. real(0)) cycle
       c3=c3+1
    end do
    outp(19)=dble(c3) !Survivors
    outp(20)=medianval(inpy(:,T1),real(0)) !Y Median
    outp(21)=medianval(inpw(:,T1),real(0)) !W median
    outp(22)=avg(l11,real(0)) !Y/N mean
    outp(23)=sqrt( variance(l11,real(outp(22)),real(0)) ) !Y/N std
    outp(37)=medianval(l11,real(0)) !NEW

    !====== now adding dynamic moments ======
    l11=inpy(:,1)/inpn(:,1)
    l12=inpy(:,2)/inpn(:,2)
    outp(24)=corr(l11,l12,real(0)) !correlation between Y/N over time - persistence
    outp(26)=corr(l12-l11,l11,real(0)) !correlation in change in Y/N and Y/N - mean reversion

    outp(34)=corr(l11,inpy(:,2)/inpy(:,1)-real(1),real(0)) !correlation between Y/N and growth in Y
    outp(35)=corr(l11,inpn(:,2)/inpn(:,1)-real(1),real(0)) !correlation between Y/N and growth in N

    l11=inpy(:,T1-1)/inpn(:,T1-1)			
    l12=inpy(:,T1)/inpn(:,T1)
    outp(25)=corr(l11,l12,real(0)) !correlation between Y/N over time - persistence
    outp(27)=corr(l12-l11,l11,real(0)) !correlation in change in Y/N and Y/N - mean reversion

    l11=inpy(:,2)/inpy(:,1)-real(1)
    outp(28)=corr(l11,inpy(:,1),real(0)) !Correlation between growth rate in Y and Y - Gibrat
    outp(29)=avg(l11,real(0)) !Average growth in Y across firms
    outp(30)=sqrt( variance(l11,real(outp(29)),real(0)) )	!Std in growth rate in Y across firms.


    c2=30
    !========== Adding a growth decomposition to auxiliary model ========
    l2=sum(inpn(:,1))
    locshare(:,1)=inpn(:,1)/l2
    l2=sum(inpn(:,T1))
    locshare(:,2)=inpn(:,T1)/l2
    locprod=dble(0)
    do j=1,N1
       if(inpn(j,1) .ne. dble(0)) locprod(j,1)=inpy(j,1)/inpn(j,1)
       if(inpn(j,T1) .ne. dble(0)) locprod(j,2)=inpy(j,T1)/inpn(j,T1)
    end do
    deltashare=locshare(:,2)-locshare(:,1)
    deltaprod=locprod(:,2)-locprod(:,1)

    avgprod(1)=sum(locprod(:,1)*locshare(:,1))
    avgprod(2)=sum(locprod(:,2)*locshare(:,2))
    decomp=dble(0)
    do j=1,N1
       if(inpy(j,1) .ne. dble(0) .and. inpy(j,T1) .ne. dble(0)) then
          decomp(1)=decomp(1)+locshare(j,1)*deltaprod(j)
          decomp(2)=decomp(2)+deltashare(j)*(locprod(j,1)-avgprod(1))
          decomp(3)=decomp(3)+deltashare(j)*deltaprod(j)
       end if
       if(inpn(j,1) .ne. real(0) .and. inpn(j,T1) .eq. real(0)) then
          decomp(4)=decomp(4)-locshare(j,1)*(locprod(j,1)-avgprod(1))
       end if
    end do
    do i=1,3
       c2=c2+1
       outp(c2)=decomp(i)/sum(decomp)
    end do
  end subroutine momvect
end module modelvar
