subroutine fiteval(length,name)
  use mpivar
  use modelvar
  use datamod
  use estimate
  use kernel
  implicit none
  integer, intent(in) :: length
  character(len=length), intent(in) :: name
  real(8) :: parest(estdim)
  integer, parameter :: figdim=50,figsimrep=5000,fignum=9
  real(8), dimension(5), parameter :: bndwdth=(/&
       &dble(50.0),&		!for Y
       &dble(25.0),&		!for W
       &dble(75.0),&		!for Y/N
       &dble( 0.2),&		!for (Delta Y)/Y
       &dble(3)/)		!for N
  real(8) :: figout(figdim,2),figin_pdf(N,1),figin_reg(N,2),mymom(momdim),mom(momdim),sortarray(figsimrep)
  real(4) :: figmat(figdim,2,fignum),figmat1(figdim,2,fignum,figsimrep),figmat_ub(figdim,2,fignum)
  integer :: i,j,err,indx(figsimrep)
!!$  interface put
!!$     module procedure put_d,put_s
!!$  end interface
  call readpar_mpi(name,parest)
  call parset(parest) !sets structural parameters given input
  call findequil(err) !determine equilibrium values for given global model parameters

  if(master) print *,'hello'
  if(master) then
     do i=1,figsimrep
        call MPI_RECV(figmat,figdim*2*fignum,MPI_REAL,MPI_ANY_SOURCE,1,MPI_COMM_WORLD,status,ierr)
        figmat1(:,:,:,i)=figmat
     end do
  else
     do i=myid,figsimrep,numprocs-1
        call simul(ydat,wdat,ndat,seed+i*2)
        figmat(:,:,1)=y_pdf()
        figmat(:,:,2)=w_pdf()
        figmat(:,:,3)=n_pdf()
        figmat(:,:,4)=yn_pdf()
        figmat(:,:,5)=gr_pdf()
        figmat(:,:,6)=yw_reg()
        figmat(:,:,7)=yny_reg()
        figmat(:,:,8)=ynn_reg()
        figmat(:,:,9)=ygr_reg()
        call MPI_SEND(figmat,figdim*2*fignum,MPI_REAL,0,1,MPI_COMM_WORLD,ierr)
     end do
  end if

  if(master) then
     figmat=zero
     do i=1,figsimrep
        figmat=figmat+figmat1(:,:,:,i)
     end do
     figmat=figmat/real(figsimrep)
     call put_s(figmat(:,:,1),'y_pdf')
     call put_s(figmat(:,:,2),'w_pdf')
     call put_s(figmat(:,:,3),'n_pdf')
     call put_s(figmat(:,:,4),'yn_pdf')
     call put_s(figmat(:,:,5),'gr_pdf')
     call put_s(figmat(:,:,6),'yw_reg')
     call put_s(figmat(:,:,7),'yny_reg')
     call put_s(figmat(:,:,8),'ynn_reg')
     call put_s(figmat(:,:,9),'ygr_reg')

     do i=1,9
        do j=1,figdim
           sortarray=figmat1(j,2,i,:)
           call sort_indx(sortarray,indx)
           figmat(j,2,i)=figmat1(j,2,i,indx(int(figsimrep*.05)))
           figmat_ub(j,2,i)=figmat1(j,2,i,indx(int(figsimrep*.95)))
        end do
     end do
     figmat(:,1,:)=figmat1(:,1,:,1)
     figmat_ub(:,1,:)=figmat1(:,1,:,1)

     call put_s(figmat(:,:,1),'y_pdf_lb')
     call put_s(figmat(:,:,2),'w_pdf_lb')
     call put_s(figmat(:,:,3),'n_pdf_lb')
     call put_s(figmat(:,:,4),'yn_pdf_lb')
     call put_s(figmat(:,:,5),'gr_pdf_lb')
     call put_s(figmat(:,:,6),'yw_reg_lb')
     call put_s(figmat(:,:,7),'yny_reg_lb')
     call put_s(figmat(:,:,8),'ynn_reg_lb')
     call put_s(figmat(:,:,9),'ygr_reg_lb')

     figmat=figmat_ub
     call put_s(figmat(:,:,1),'y_pdf_ub')
     call put_s(figmat(:,:,2),'w_pdf_ub')
     call put_s(figmat(:,:,3),'n_pdf_ub')
     call put_s(figmat(:,:,4),'yn_pdf_ub')
     call put_s(figmat(:,:,5),'gr_pdf_ub')
     call put_s(figmat(:,:,6),'yw_reg_ub')
     call put_s(figmat(:,:,7),'yny_reg_ub')
     call put_s(figmat(:,:,8),'ynn_reg_ub')
     call put_s(figmat(:,:,9),'ygr_reg_ub')
  end if

  call MPI_BARRIER(MPI_COMM_WORLD,ierr)
CONTAINS
  function y_pdf()
    implicit none
    real(8) :: y_pdf(figdim,2)
    figin_pdf(:,1)=ydat(:,1)/dble(100)
    call krnlest(figin_pdf,y_pdf,bndwdth(1),0,dble(5),dble(1000))
    y_pdf(:,2)=y_pdf(:,2)*dble(100)
  end function y_pdf
  function w_pdf()
    implicit none
    real(8) :: w_pdf(figdim,2)
    figin_pdf(:,1)=wdat(:,1)/dble(100)
    call krnlest(figin_pdf,w_pdf,bndwdth(2),0,dble(0),dble(500))
    w_pdf(:,2)=w_pdf(:,2)*dble(100)
  end function w_pdf
  function n_pdf()
    implicit none
    real(8) :: n_pdf(figdim,2)
    figin_pdf(:,1)=ndat(:,1)
    call krnlest(figin_pdf,n_pdf,bndwdth(5),0,dble(0),dble(250))
    n_pdf(:,2)=n_pdf(:,2)*dble(100)
  end function n_pdf
  function yn_pdf()
    implicit none
    real(8) :: yn_pdf(figdim,2)
    figin_pdf(:,1)=ydat(:,1)/ndat(:,1)
    call krnlest(figin_pdf,yn_pdf,bndwdth(3),0,dble(5),dble(1000))
    yn_pdf(:,2)=yn_pdf(:,2)*dble(100)
  end function yn_pdf
  function gr_pdf()
    implicit none
    real(8) :: gr_pdf(figdim,2)
    figin_pdf(:,1)=(ydat(:,2)-ydat(:,1))/ydat(:,1)
    call krnlest(figin_pdf,gr_pdf,bndwdth(4),0,dble(-1),dble(2))
    gr_pdf(:,2)=gr_pdf(:,2)*dble(100)
  end function gr_pdf
  function yw_reg()
    implicit none
    real(8) :: yw_reg(figdim,2)
    figin_reg(:,1)=ydat(:,1)/dble(100)
    figin_reg(:,2)=wdat(:,1)/dble(100)
    call krnlest(figin_reg,yw_reg,bndwdth(1),0,dble(0),dble(1000))
  end function yw_reg
  function yny_reg()
    implicit none
    real(8) :: yny_reg(figdim,2)
    figin_reg(:,1)=ydat(:,1)/ndat(:,1)
    figin_reg(:,2)=ydat(:,1)/dble(100)
    call krnlest(figin_reg,yny_reg,bndwdth(3),0,dble(5),dble(1000))
    yny_reg(:,2)=yny_reg(:,2)/dble(10)
  end function yny_reg
  function ynn_reg()
    implicit none
    real(8) :: ynn_reg(figdim,2)
    figin_reg(:,1)=ydat(:,1)/ndat(:,1)
    figin_reg(:,2)=ndat(:,1)
    call krnlest(figin_reg,ynn_reg,bndwdth(3),0,dble(5),dble(1000))
  end function ynn_reg
  function ygr_reg()
    implicit none
    real(8) :: ygr_reg(figdim,2)
    figin_reg(:,1)=ydat(:,1)/dble(100)
    figin_reg(:,2)=(ydat(:,2)-ydat(:,1))/ydat(:,1)
    call krnlest(figin_reg,ygr_reg,bndwdth(1),0,dble(5),dble(1000))
  end function ygr_reg
  subroutine put_d(inp,name)
    implicit none
    real(8),intent(in) :: inp(figdim,2)
    character(*), intent(in) :: name
    integer :: ii
    open(10,file=root//outdir//'figser//'//name//'.csv')
    do ii=1,figdim
       write(10,'(f20.10,", ",f20.10)'),inp(ii,1),inp(ii,2)
    end do
    close(10)
  end subroutine put_d
  subroutine put_s(inp,name)
    implicit none
    real(4),intent(in) :: inp(figdim,2)
    character(*), intent(in) :: name
    integer :: ii
    open(10,file=root//outdir//'figser//'//name//'.csv')
    do ii=1,figdim
       write(10,'(f20.10,", ",f20.10)'),inp(ii,1),inp(ii,2)
    end do
    close(10)
  end subroutine put_s
end subroutine fiteval

subroutine pareval(length,name)
  use mpivar
  use modelvar
  use datamod
  use estimate
  implicit none
  integer, intent(in) :: length
  character(len=length), intent(in) :: name
  integer, parameter :: pardim=estdim+12+11*en
  real(8) :: parest(estdim),parout(pardim-estdim),bootmat(estdim,momentmcdim),fullpar(pardim,1),parmat(pardim,momentmcdim),vc(pardim,pardim),l1(pardim,1),fval0,mypar(pardim)
  integer :: i,j,lb,ub

  call readpar_mpi(name,parest)
  if(master) then
     open(10,file=root//outdir//'bootres_'//name//'.csv',status='old')
     read(10,*) ((bootmat(i,j), i=1,estdim), j=1,momentmcdim)
     forall(i=1:estdim,j=1:momentmcdim,bootmat(i,j) .le. lo(i)) bootmat(i,j)=lo(i)+1.0e-10 !fix potential problem that written output may have been rounded down to lower bound
     close(10)
  end if
  call MPI_BCAST(bootmat,estdim*momentmcdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)

!==================
! Counter factuals
!!$  sigma=one
!!$  parest(4)=zero
!!$  parest(6)=parest(4)
!!$  parest(8)=zero
!!$  parest(9)=zero
!!$  bootmat(7,:)=one
!!$  bootmat(9,:)=zero
!!$  bootmat(10,:)=zero
!===================
  vc=zero

  ver='3t'
  estmin=exp(varhuge)
  call esteval(parest,parout)
  fullpar(1:estdim,1)=parest
  fullpar(estdim+1:pardim,1)=parout
  parmat=zero
  if(master) then
     do i=1,momentmcdim
        call MPI_RECV(mypar,pardim,MPI_DOUBLE_PRECISION,MPI_ANY_SOURCE,1,MPI_COMM_WORLD,status,ierr)
        parmat(:,i)=mypar
     end do
  else
     do i=myid,momentmcdim,numprocs-1
        call esteval(bootmat(:,i),parout)
        mypar(1:estdim)=bootmat(:,i)
        mypar(estdim+1:pardim)=parout
        call MPI_SEND(mypar,pardim,MPI_DOUBLE_PRECISION,0,1,MPI_COMM_WORLD,ierr)
     end do
  end if
  call MPI_BCAST(parmat,pardim*momentmcdim,MPI_DOUBLE_PRECISION,0,MPI_COMM_WORLD,ierr)

  if(master) then
     do i=1,momentmcdim
        l1(:,1)=parmat(:,i)-sum(parmat,dim=2)/dble(momentmcdim)
        vc=vc+matmul(l1,transpose(l1))
     end do
     vc=vc/dble(momentmcdim-1)
     call tab1
     call tab4
  end if
  call MPI_BARRIER(MPI_COMM_WORLD,ierr)

  simrep=20*numprocs
  ver='3t'
  estmin=exp(varhuge)
  call simdist(parest,fval0)

  if(master) call tab2
  if(master) call tab3

CONTAINS
  subroutine tab1
    implicit none
    integer :: i,indx(en)
    real(8) :: typevec(en)
    character(*), parameter :: empty='('//char(47+en)//'(" &")," \\")'
    character*13 :: form1(0:en)
    character*19 :: form2(0:en)
    form1(0)='('
    do i=1,en-1
       form1(i)='f10.4,"  & ",'
    end do
    form1(en)='f10.4,"  \\")'
    form2(0)='('
    do i=1,en-1
       form2(i)='"(",f9.4,")"," & ",'
    end do
    form2(en)='"(",f9.4,")"," \\")'
    open(10,file=root//outdir//'tab1_'//name//'.tex',status='replace')
!!$    open(10,file=root//outdir//"simann/tab1.tex",status='replace')
!!$    write(10,'("\begin{table}[t]\caption{Parameter Estimates}\label{tab1}\vspace{.5em}")')
    write(10,'("\center{")')
    write(10,'("\begin{tabular}{ld{5.6}@{\hspace{4em}}l",$)')
    do i=1,en-1
       write(10,'("d{5.6}",$)')
    end do
    write(10,'("d{5.6}}")')
    write(10,'("\hline\hline \\[-1ex]")')
    write(10,'(" & & &",$)')
    do i=1,en-1
       write(10,10),'$\tau='//char(48+i)//'$'
    end do
    write(10,20),'$\tau='//char(48+en)//'$'
    write(10,'("\hline \T")')

    write(10,210),fullpar(1,1);write(10,form1) (fullpar(estdim+18+(i-1)*11,1), i=1,en)
    write(10,200),'$c_0/Z$',sqrt(vc(1,1)),'$\phi_{\tau}$';write(10,form2) (sqrt(vc(estdim+18+(i-1)*11,estdim+18+(i-1)*11)), i=1,en)

    write(10,210),fullpar(2,1);write(10,form1) (fullpar(estdim+20+(i-1)*11,1), i=1,en)
    write(10,200),'$c_1$',sqrt(vc(2,2)),'$\phi^{\ast}_{\tau}$';write(10,form2) (sqrt(vc(estdim+20+(i-1)*11,estdim+20+(i-1)*11)), i=1,en)
    
    typevec=fullpar(10:9+en,1)
    call sort_indx(typevec,indx)
    write(10,210),fullpar(estdim,1);write(10,form1) (typevec(indx(i)), i=1,en)
    do i=10,9+en
      typevec(i-9)=sqrt(vc(i,i))
    end do
    write(10,200),'$\kappa$',sqrt(vc(estdim,estdim)),'$\xi_{\tau}$';write(10,form2) (typevec(indx(i)), i=1,en)

    typevec=fullpar(5,1)
    write(10,210),fullpar(4,1);write(10,form1) (typevec(i), i=1,en)
    typevec=sqrt(vc(5,5))
    write(10,200),'$Z$',sqrt(vc(4,4)),'$\beta_q$';write(10,form2) (typevec(i), i=1,en)

    write(10,210),fullpar(7,1);write(10,form1) (fullpar(estdim+16+(i-1)*11,1), i=1,en)
    write(10,200),'$\beta_Z$',sqrt(vc(7,7)),'$\gamma_{\tau}$';write(10,form2) (sqrt(vc(estdim+16+(i-1)*11,estdim+16+(i-1)*11)), i=1,en)

    write(10,210),fullpar(6,1);write(10,form1) (fullpar(estdim+17+(i-1)*11,1), i=1,en)
    write(10,200),'$o_Z$',sqrt(vc(6,6)),'$K_{\tau}$';write(10,form2) (sqrt(vc(estdim+17+(i-1)*11,estdim+17+(i-1)*11)), i=1,en)

    write(10,210),fullpar(3,1);write(10,form1) (fullpar(estdim+11+(i-1)*11,1), i=1,en)
    write(10,200),'$\eta$',sqrt(vc(3,3)),'$v_{\tau}$';write(10,form2) (sqrt(vc(estdim+11+(i-1)*11,estdim+11+(i-1)*11)), i=1,en)

    write(10,210),fullpar(8,1);write(10,form1) (fullpar(estdim+10+(i-1)*11,1), i=1,en)
    write(10,200),'$\sigma^2_Y$',sqrt(vc(8,8)),'$\pi_{\tau}$';write(10,form2) (sqrt(vc(estdim+10+(i-1)*11,estdim+10+(i-1)*11)), i=1,en)

    write(10,210),fullpar(9,1);write(10,form1) (fullpar(estdim+13+(i-1)*11,1), i=1,en)
    write(10,200),'$\sigma^2_W$',sqrt(vc(10,10)),'Med$[\tilde{q}_{\tau}]$';write(10,form2) (sqrt(vc(estdim+13+(i-1)*11,estdim+13+(i-1)*11)), i=1,en)

    write(10,210),fullpar(estdim+8,1);write(10,form1) (fullpar(estdim+19+(i-1)*11,1), i=1,en)
    write(10,200),'$\delta$',sqrt(vc(estdim+8,estdim+8)),'E$[\tilde{k}_{\tau}]$';write(10,form2) (sqrt(vc(estdim+19+(i-1)*11,estdim+19+(i-1)*11)), i=1,en)

    write(10,210),fullpar(estdim+5,1);write(10,empty)
    write(10,200),'$\mu$',sqrt(vc(estdim+5,estdim+5)),'';write(10,empty)

    write(10,210),fullpar(estdim+6,1);write(10,empty)
    write(10,200),'$\ell$',sqrt(vc(estdim+6,estdim+6)),'';write(10,empty)

    write(10,210),fullpar(estdim+1,1);write(10,empty)
    write(10,200),'$g$',sqrt(vc(estdim+1,estdim+1)),'';write(10,empty)

    write(10,'("\raisebox{0.0ex}[0pt]{                  $\sigma$} \vspace{.3em} & ",f10.4," & & ", $)'),sigma;write(10,empty)

    write(10,'("\hline")')
    write(10,'("\end{tabular}}")')
    close(10)

10  format("\multicolumn{1}{c}{",A20,"} & ",$)
20  format("\multicolumn{1}{c}{",A20,"} \\")
200 format("\raisebox{1.5ex}[0pt]{",A30,"}\vspace{.3em} & (",f10.4,") & ","\raisebox{1.5ex}[0pt]{",A30,"} & ",$)
201 format("\B \raisebox{1.5ex}[0pt]{",A30,"}\vspace{.0em} & (",f10.4,") & ","\raisebox{1.5ex}[0pt]{",A30,"} & ",$)
210 format(67(" "),"&  ",f10.4,"  & ",54(" "),"& ",$)
  end subroutine tab1

  subroutine tab2
    implicit none
    real(8) :: l5
    integer, parameter :: xx=33
    open(10,file=root//outdir//'tab2_'//name//'.tex',status='replace')
!!$    open(10,file=root//outdir//"simann/tab2.tex",status='replace')
!!$    write(10,'("\begin{table}[t]\caption{Data Moments}\label{tab2}\vspace{.5em}")')
    write(10,'("\center{")')
    write(10,'("\begin{tabular}{ld{5.5}d{5.5}@{\hspace{1.5em}}ld{4.4}d{4.4}}")')
    write(10,'("\hline\hline \\[-1ex]")')
    write(10,'(" & \multicolumn{1}{l}{\hspace{1.5em}1992} & \multicolumn{1}{l}{\hspace{1.5em}1997} & & \multicolumn{1}{l}{\hspace{1.5em}1992} & \multicolumn{1}{l}{\hspace{1.5em}1996} \\")')
    write(10,'("\hline \T")')

    write(10,305),dble(N),datmom(19),datmom(24),datmom(25)
    write(10,302),'Survivors','\multicolumn{1}{c}{--}',sqrt(datavarcov(19,19)),'Cor$[\frac Y{N^\ast},\frac{Y_{+1}}{N^\ast_{+1}}]$',sqrt(datavarcov(24,24)),sqrt(datavarcov(25,25))

    write(10,305),datmom(1),datmom(12),datmom(26),datmom(27)
    write(10,306),'E$[Y]$',sqrt(datavarcov(1,1)),sqrt(datavarcov(12,12)),'Cor$[\frac Y{N^\ast},\Delta\frac Y{N^\ast}]$',sqrt(datavarcov(26,26)),sqrt(datavarcov(27,27))

    write(10,301),datmom(8),datmom(20),datmom(34)
    write(10,300),'Med$[Y]$',sqrt(datavarcov(8,8)),sqrt(datavarcov(20,20)),'Cor$[\frac Y{N^\ast},\frac{\Delta Y}Y]$',sqrt(datavarcov(34,34))

    write(10,301),datmom(3),datmom(14),datmom(35)
    write(10,300),'Std$[Y]$',sqrt(datavarcov(3,3)),sqrt(datavarcov(14,14)),'Cor$[\frac Y{N^\ast},\frac{\Delta N^\ast}{N^\ast}]$',sqrt(datavarcov(35,35))

    write(10,301),datmom(2),datmom(13),datmom(29)
    write(10,300),'E$[W]$',sqrt(datavarcov(2,2)),sqrt(datavarcov(13,13)),'E$[\frac{\Delta Y}Y]$',sqrt(datavarcov(29,29))

    write(10,301),datmom(9),datmom(21),datmom(30)
    write(10,300),'Med$[W]$',sqrt(datavarcov(9,9)),sqrt(datavarcov(21,21)),'Std$[\frac{\Delta Y}Y]$',sqrt(datavarcov(30,30))

    write(10,301),datmom(4),datmom(15),datmom(28)
    write(10,300),'Std$[W]$',sqrt(datavarcov(4,4)),sqrt(datavarcov(15,15)),'Cor$[\frac{\Delta Y}Y,Y]$',sqrt(datavarcov(28,28))

    write(10,301),datmom(10),datmom(22),datmom(31)
    write(10,300),'E$[\frac Y{N^\ast}]$',sqrt(datavarcov(10,10)),sqrt(datavarcov(22,22)),'Within',sqrt(datavarcov(31,31))

    write(10,301),datmom(36),datmom(37),datmom(32)
    write(10,300),'Med$[\frac Y{N^\ast}]$',sqrt(datavarcov(36,36)),sqrt(datavarcov(37,37)),'Between',sqrt(datavarcov(32,32))

    write(10,301),datmom(11),datmom(23),datmom(33)
    write(10,300),'Std$[\frac Y{N^\ast}]$',sqrt(datavarcov(11,11)),sqrt(datavarcov(23,23)),'Cross',sqrt(datavarcov(33,33))

    l5=dble(0.0)
    do c1=xx-2,xx
       l5=l5+datavarcov(c1,c1)
    end do
    l5=l5+2.0d0*(datavarcov(xx,xx-1)+datavarcov(xx,xx-2)+datavarcov(xx-1,xx-2))

    write(10,301),datmom(5),datmom(16),one-sum(datmom(31:33))
    write(10,300),'Cor$[Y,W]$',sqrt(datavarcov(5,5)),sqrt(datavarcov(16,16)),'Exit',sqrt(l5)

    write(10,304),datmom(6),datmom(17)
    write(10,303),'Cor$[\frac Y{N^\ast},N^\ast]$',sqrt(datavarcov(6,6)),sqrt(datavarcov(17,17))

    write(10,304),datmom(7),datmom(18)
    write(10,303),'Cor$[\frac Y{N^\ast},Y]$',sqrt(datavarcov(7,7)),sqrt(datavarcov(18,18))

    write(10,'("\hline")')
    write(10,'("\end{tabular}}")')
!!$    write(10,'("\end{table}")')
    close(10)

300 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} & (",f8.3,") & (",f8.3,") & \raisebox{1.5ex}[0pt]{",A55,"} & (",f8.3,") & \\")
301 format(92(" "),"& ",f9.3,"  & ",f9.3,"  & ",79(" "),"& ",f9.3," &  \\")
302 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} &  ",A30," & (",f8.3,") & \raisebox{1.5ex}[0pt]{",A55,"} & (",f8.3,") & (",f8.3,") \\")
303 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} & (",f8.3,") & (",f8.3,") & & & \\")
304 format(92(" "),"& ",f9.3,"  & ",f9.3,"  & & & \\")
305 format(92(" "),"& ",f9.3,"  & ",f9.3,"  & ",79(" "),"& ",f9.3,"& ",f9.3,"  \\")
306 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} & (",f8.3,") & (",f8.3,") & \raisebox{1.5ex}[0pt]{",A55,"} & (",f8.3,") & (",f8.3,") \\")
  end subroutine tab2

  subroutine tab3
    implicit none
    real(8) :: l5
    integer, parameter :: xx=33
    open(10,file=root//outdir//'tab3_'//name//'.tex',status='replace')
!!$    open(10,file=root//outdir//"simann/tab3.tex",status='replace')
!!$    write(10,'("\begin{table}[t]\caption{Model Fit}\label{tab3}\vspace{.5em}")')
    write(10,'("\center{")')
    write(10,'("\begin{tabular}{ld{5.5}d{5.5}@{\hspace{1.5em}}ld{4.4}d{4.4}}")')
    write(10,'("\hline\hline \\[-1ex]")')
    write(10,'(" & \multicolumn{1}{l}{\hspace{1.5em}1992} & \multicolumn{1}{l}{\hspace{1.5em}1997} & & \multicolumn{1}{l}{\hspace{1.5em}1992} & \multicolumn{1}{l}{\hspace{1.5em}1996} \\")')
    write(10,'("\hline \T")')

    write(10,305),dble(N),datmom(19),datmom(24),datmom(25)
    write(10,306),'Survivors',dble(N),simmom(19),'Cor$[\frac Y{N^\ast},\frac{Y_{+1}}{N^\ast_{+1}}]$',simmom(24),simmom(25)

    write(10,305),datmom(1),datmom(12),datmom(26),datmom(27)
    write(10,306),'E$[Y]$',simmom(1),simmom(12),'Cor$[\frac Y{N^\ast},\Delta\frac Y{N^\ast}]$',simmom(26),simmom(27)

    write(10,301),datmom(8),datmom(20),datmom(34)
    write(10,300),'Med$[Y]$',simmom(8),simmom(20),'Cor$[\frac Y{N^\ast},\frac{\Delta Y}Y]$',simmom(34)

    write(10,301),datmom(3),datmom(14),datmom(35)
    write(10,300),'Std$[Y]$',simmom(3),simmom(14),'Cor$[\frac Y{N^\ast},\frac{\Delta N^\ast}{N^\ast}]$',simmom(35)

    write(10,301),datmom(2),datmom(13),datmom(29)
    write(10,300),'E$[W]$',simmom(2),simmom(13),'E$[\frac{\Delta Y}Y]$',simmom(29)

    write(10,301),datmom(9),datmom(21),datmom(30)
    write(10,300),'Med$[W]$',simmom(9),simmom(21),'Std$[\frac{\Delta Y}Y]$',simmom(30)

    write(10,301),datmom(4),datmom(15),datmom(28)
    write(10,300),'Std$[W]$',simmom(4),simmom(15),'Cor$[\frac{\Delta Y}Y,Y]$',simmom(28)

    write(10,301),datmom(10),datmom(22),datmom(31)
    write(10,300),'E$[\frac Y{N^\ast}]$',simmom(10),simmom(22),'Within',simmom(31)
    write(6,'(f10.4)'),simmom(31)

    write(10,301),datmom(36),datmom(37),datmom(32)
    write(10,300),'Med$[\frac Y{N^\ast}]$',simmom(36),simmom(37),'Between',simmom(32)
    write(6,'(f10.4)'),simmom(32)

    write(10,301),datmom(11),datmom(23),datmom(33)
    write(10,300),'Std$[\frac Y{N^\ast}]$',simmom(11),simmom(23),'Cross',simmom(33)
    write(6,'(f10.4)'),simmom(33)

    write(10,301),datmom(5),datmom(16),one-sum(datmom(31:33))
    write(10,300),'Cor$[Y,W]$',simmom(5),simmom(16),'Exit',one-sum(simmom(31:33))

    write(10,304),datmom(6),datmom(17)
    write(10,303),'Cor$[\frac Y{N^\ast},N^\ast]$',simmom(6),simmom(17)

    write(10,304),datmom(7),datmom(18)
    write(10,303),'Cor$[\frac Y{N^\ast},Y]$',simmom(7),simmom(18)

    write(10,'("\hline")')
    write(10,'("\end{tabular}}")')
!!$    write(10,'("\end{table}")')
    close(10)

300 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} & ",f9.3,"  & ",f9.3,"  & \raisebox{1.5ex}[0pt]{",A55,"} & ",f9.3,"  & \\")
301 format(92(" "),"& ",f9.3,"  & ",f9.3,"  & ",79(" "),"& ",f9.3," &  \\")
302 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} &  ",A30," & ",f9.3,"  & \raisebox{1.5ex}[0pt]{",A55,"} & ",f9.3,"  & ",f9.3,"  \\")
303 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} & ",f9.3,"  & ",f9.3,"  & & & \\")
304 format(92(" "),"& ",f9.3,"  & ",f9.3,"  & & & \\")
305 format(92(" "),"& ",f9.3,"  & ",f9.3,"  & ",79(" "),"& ",f9.3,"& ",f9.3,"  \\")
306 format("\raisebox{1.5ex}[0pt]{",A55,"}\vspace{.3em} & ",f9.3,"  & ",f9.3,"  & \raisebox{1.5ex}[0pt]{",A55,"} & ",f9.3,"  & ",f9.3,"  \\")

  end subroutine tab3

  subroutine tab4
    implicit none
    real(8) :: l5
    integer, parameter :: xx=33
    open(10,file=root//outdir//'tab4_'//name//'.tex',status='replace')
!!$    open(10,file=root//outdir//"simann/tab4.tex",status='replace')
!!$    write(10,'("\begin{table}[t]\caption{Productivity Growth Rate and Its Components}\label{tab3}\vspace{.5em}")')
    write(10,'("\center{")')
    write(10,'("\begin{tabular}{ld{2.5}d{2.5}}")')
    write(10,'("\hline\hline \\[-1ex]")')
    write(10,'(" & \multicolumn{1}{c}{Point Estimate} & \multicolumn{1}{c}{Fraction of $g$} \\")')
    write(10,'("\hline \T")')

    write(10,400),fullpar(estdim+1,1),one
    write(10,'("\vspace{1em} ",$)');write(10,402),'$g$',sqrt(vc(estdim+1,estdim+1)),'\multicolumn{1}{c}{---}'

    write(10,400),fullpar(pardim-2,1),fullpar(estdim+2,1)
    write(10,401),'Entry/exit',sqrt(vc(pardim-2,pardim-2)),sqrt(vc(estdim+2,estdim+2))

    write(10,400),fullpar(pardim-1,1),fullpar(estdim+3,1)
    write(10,401),'Residual',sqrt(vc(pardim-1,pardim-1)),sqrt(vc(estdim+3,estdim+3))

    write(10,400),fullpar(pardim,1),fullpar(estdim+4,1)
    write(10,401),'Selection',sqrt(vc(pardim,pardim)),sqrt(vc(estdim+4,estdim+4))

    write(10,'("\hline")')
    write(10,'("\end{tabular}}")')
!!$    write(10,'("\end{table}")')
    close(10)


400 format(67(" "),"&  ",f10.4,"  &  ",f10.4,"  \\")
401 format("\raisebox{1.5ex}[0pt]{",A30,"}\vspace{.3em} & (",f10.4,") & (",f10.4,") \\")
402 format("\raisebox{1.5ex}[0pt]{",A30,"}\vspace{.3em} & ( ",f10.4,") &  ",A30,"  \\")
  end subroutine tab4

end subroutine pareval
