!===============================================================================================================================
!===============================================================================================================================
module moments !This module makes basic statistical moments available
  implicit none
  interface avg
     module procedure avg_r,avg_d
  end interface
  interface variance
     module procedure variance_r,variance_d
  end interface
  interface corr
     module procedure corr_r,corr_d,corr_i
  end interface
  interface medianval
     module procedure medianval_r,medianval_d
  end interface
  interface swap
     module procedure swap_d,masked_swap_ds,swap_i,masked_swap_is
  end interface
  interface sort
     module procedure sort_i,sort_d
  end interface
contains
  function avg_r(inp,miss)
    implicit none
    real(4) avg_r
    real(4), dimension(:), intent(in) :: inp
    real(4), optional, intent(in) :: miss !optional mask where user can identify values in vector that should be treated as missing.
    real(4) :: zero,neg_inf,pos_inf
    real(8) :: l1
    integer :: i,j
    zero=real(0)
    pos_inf=real(1)/zero
    neg_inf=real(-1)/zero
    j=0
    l1=real(0)
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(inp(i) .eq. pos_inf .or. inp(i) .eq. neg_inf .or. isnan(inp(i))) cycle
       l1=l1+inp(i)
       j=j+1
    end do
    avg_r=l1/dble(j)
  end function avg_r

  function avg_d(inp,miss)
    implicit none
    real(8) avg_d
    real(8), dimension(:), intent(in) :: inp
    real(8), optional, intent(in) :: miss
    real(8) :: zero,neg_inf,pos_inf
    integer :: i,j
    zero=dble(0)
    pos_inf=dble(1)/zero
    neg_inf=dble(-1)/zero
    j=0
    avg_d=dble(0)
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(inp(i) .eq. pos_inf .or. inp(i) .eq. neg_inf .or. isnan(inp(i))) cycle
       avg_d=avg_d+inp(i)
       j=j+1
    end do
    avg_d=avg_d/dble(j)
  end function avg_d

  function variance_r(inp,inp_avg,miss)
    implicit none
    real(4) variance_r
    real(4), dimension(:), intent(in) :: inp
    real(4), optional, intent(in) :: inp_avg !It's optional to provide the average - speeds up the calc if it's already known.
    real(4), optional, intent(in) :: miss !User can identify values in inp that should be treated as missing.
    real(4) :: zero, neg_inf,pos_inf,loc_avg
    real(8) :: l1
    integer :: i,j
    if(present(inp_avg)) then
       loc_avg=inp_avg
    else
       loc_avg=avg(inp)
    end if
    zero=real(0)
    pos_inf=real(1)/zero
    neg_inf=real(-1)/zero
    j=0
    l1=real(0)
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(inp(i) .eq. pos_inf .or. inp(i) .eq. neg_inf .or. isnan(inp(i))) cycle
       l1=l1+inp(i)*inp(i)
       j=j+1
    end do
    variance_r=l1/dble(j)-loc_avg*loc_avg
  end function variance_r

  function variance_d(inp,inp_avg,miss)
    implicit none
    real(8) variance_d
    real(8), dimension(:), intent(in) :: inp
    real(8), optional, intent(in) :: inp_avg !It's optional to provide the average - speeds up the calc if it's already known.
    real(8), optional, intent(in) :: miss !User can identify values in inp that should be treated as missing
    real(8) :: zero, neg_inf,pos_inf,loc_avg
    integer :: i,j
    if(present(inp_avg)) then
       loc_avg=inp_avg
    else
       loc_avg=avg(inp)
    end if
    zero=dble(0)
    pos_inf=dble(1)/zero
    neg_inf=dble(-1)/zero
    j=0
    variance_d=dble(0)
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(inp(i) .eq. pos_inf .or. inp(i) .eq. neg_inf .or. isnan(inp(i))) cycle
       variance_d=variance_d+inp(i)*inp(i)
       j=j+1
    end do
    variance_d=variance_d/dble(j)-loc_avg*loc_avg
  end function variance_d

  function corr_i(inp1,inp2,miss)
    implicit none
    real(8) corr_i
    integer(1), dimension(:), intent(in) :: inp1, inp2
    logical, optional, dimension(:), intent(in) :: miss
    real(8) :: x, y, xx, yy, xy
    integer(1) :: zero, neg_inf, pos_inf
    integer :: i,j
    if(size(inp1) .ne. size(inp2)) pause 'Failure (CORR_R): input arrays must be of same length.'
    zero=0
    pos_inf=real(1)/zero
    neg_inf=real(-1)/zero
    x=zero
    y=zero
    xx=zero
    yy=zero
    xy=zero
    j=0
    do i=1,size(inp1)
       if(miss(i)) then
          x=x+inp1(i)
          y=y+inp2(i)
          xx=xx+dble(inp1(i))*dble(inp1(i))
          yy=yy+dble(inp2(i))*dble(inp2(i))
          xy=xy+dble(inp1(i))*dble(inp2(i))
          j=j+1
       end if
    end do
    corr_i=dble(xy-x*y/dble(j))/sqrt((xx-x*x/dble(j))*(yy-y*y/dble(j)))
  end function corr_i

  function corr_r(inp1,inp2,miss)
    implicit none
    real(4) corr_r
    real(4), dimension(:), intent(in) :: inp1, inp2
    real(4), optional, intent(in) :: miss
    real(8) :: x, y, xx, yy, xy
    real(4) :: zero, neg_inf, pos_inf
    integer :: i,j
    if(size(inp1) .ne. size(inp2)) pause 'Failure (CORR_R): input arrays must be of same length.'
    zero=real(0)
    pos_inf=real(1)/zero
    neg_inf=real(-1)/zero
    x=zero
    y=zero
    xx=zero
    yy=zero
    xy=zero
    j=0
    do i=1,size(inp1)
       if(present(miss)) then
          if(inp1(i) .eq. miss .or. inp2(i) .eq. miss) cycle
       end if
       if(inp1(i) .eq. pos_inf .or. inp1(i) .eq. neg_inf .or. isnan(inp1(i))) cycle
       if(inp2(i) .eq. pos_inf .or. inp2(i) .eq. neg_inf .or. isnan(inp2(i))) cycle
       x=x+inp1(i)
       y=y+inp2(i)
       xx=xx+dble(inp1(i))*dble(inp1(i))
       yy=yy+dble(inp2(i))*dble(inp2(i))
       xy=xy+dble(inp1(i))*dble(inp2(i))
       j=j+1
    end do
    corr_r=dble(xy-x*y/dble(j))/sqrt((xx-x*x/dble(j))*(yy-y*y/dble(j)))
  end function corr_r

  function corr_d(inp1,inp2,miss)
    implicit none
    real(8) corr_d
    real(8), dimension(:), intent(in) :: inp1, inp2
    real(8), optional, intent(in) :: miss
    real(8) :: x, y, xx, yy, xy, zero, neg_inf, pos_inf
    integer :: i,j
    if(size(inp1) .ne. size(inp2)) pause 'Failure (CORR_D): input arrays must be of same length.'
    zero=dble(0)
    pos_inf=dble(1)/zero
    neg_inf=dble(-1)/zero
    x=zero
    y=zero
    xx=zero
    yy=zero
    xy=zero
    j=0
    do i=1,size(inp1)
       if(present(miss)) then
          if(inp1(i) .eq. miss .or. inp2(i) .eq. miss) cycle
       end if
       if(inp1(i) .eq. pos_inf .or. inp1(i) .eq. neg_inf .or. isnan(inp1(i))) cycle
       if(inp2(i) .eq. pos_inf .or. inp2(i) .eq. neg_inf .or. isnan(inp2(i))) cycle
       x=x+inp1(i)
       y=y+inp2(i)
       xx=xx+inp1(i)*inp1(i)
       yy=yy+inp2(i)*inp2(i)
       xy=xy+inp1(i)*inp2(i)
       j=j+1
    end do
    corr_d=(xy-x*y/dble(j))/dsqrt((xx-x*x/dble(j))*(yy-y*y/dble(j)))
  end function corr_d

  function medianval_r(inp,miss)
    implicit none
    real(4) medianval_r
    real(4), dimension(:), intent(in) :: inp
    real(4), optional, intent(in) :: miss
    real(8), allocatable :: l1(:)
    integer :: i,j
    if(size(inp) .eq. 1) then
       medianval_r=inp(1)
       return
    end if
    j=0
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(isnan(inp(i))) cycle
       j=j+1
    end do
    allocate(l1(j))
    j=0
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(isnan(inp(i))) cycle
       j=j+1
       l1(j)=inp(i)
    end do
    call sort(l1)
    if(mod(real(j),real(2)) .ne. real(0)) then
       medianval_r=l1((j+1)/2)
    else
       medianval_r=(l1(j/2)+l1(j/2+1))/real(2)
    end if
  end function medianval_r

  function medianval_d(inp,miss)
    implicit none
    real(8) medianval_d
    real(8), dimension(:), intent(in) :: inp
    real(8), optional, intent(in) :: miss
    real(8), allocatable :: l1(:)
    integer :: i,j
    if(size(inp) .eq. 1) then
       medianval_d=inp(1)
       return
    end if
    j=0
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(isnan(inp(i))) cycle
       j=j+1
    end do
    allocate(l1(j))
    j=0
    do i=1,size(inp)
       if(present(miss)) then
          if(inp(i) .eq. miss) cycle
       end if
       if(isnan(inp(i))) cycle
       j=j+1
       l1(j)=inp(i)
    end do
    call sort(l1)
    if(mod(real(j),real(2)) .ne. real(0)) then
       medianval_d=l1((j+1)/2)
    else
       medianval_d=(l1(j/2)+l1(j/2+1))/dble(2)
    end if
  end function medianval_d

  SUBROUTINE sort_d(arr)
    IMPLICIT NONE
    REAL(8), DIMENSION(:), INTENT(INOUT) :: arr
    INTEGER, PARAMETER :: NN=15, NSTACK=50
    REAL(8) :: a
    INTEGER :: n,k,i,j,jstack,l,r
    INTEGER, DIMENSION(NSTACK) :: istack
    n=size(arr)
    jstack=0
    l=1
    r=n
    do
       if (r-l < NN) then
          do j=l+1,r
             a=arr(j)
             do i=j-1,l,-1
                if (arr(i) <= a) exit
                arr(i+1)=arr(i)
             end do
             arr(i+1)=a
          end do
          if (jstack == 0) RETURN
          r=istack(jstack)
          l=istack(jstack-1)
          jstack=jstack-2
       else
          k=(l+r)/2
          call swap(arr(k),arr(l+1))
          call swap(arr(l),arr(r),arr(l)>arr(r))
          call swap(arr(l+1),arr(r),arr(l+1)>arr(r))
          call swap(arr(l),arr(l+1),arr(l)>arr(l+1))
          i=l+1
          j=r
          a=arr(l+1)
          do
             do
                i=i+1
                if (arr(i) >= a) exit
             end do
             do
                j=j-1
                if (arr(j) <= a) exit
             end do
             if (j < i) exit
             call swap(arr(i),arr(j))
          end do
          arr(l+1)=arr(j)
          arr(j)=a
          jstack=jstack+2
          if (jstack > NSTACK) pause 'sort: NSTACK too small'
          if (r-i+1 >= j-l) then
             istack(jstack)=r
             istack(jstack-1)=i
             r=j-1
          else
             istack(jstack)=j-1
             istack(jstack-1)=l
             l=i
          end if
       end if
    end do
  END SUBROUTINE sort_d

  SUBROUTINE sort_i(arr,order)
    IMPLICIT NONE
    INTEGER, DIMENSION(:), INTENT(INOUT) :: arr
    integer, dimension(size(arr)), intent(out) :: order
    INTEGER, PARAMETER :: NN=15, NSTACK=50
    INTEGER :: a
    INTEGER :: n,k,i,j,jstack,l,r,order_scal
    INTEGER, DIMENSION(NSTACK) :: istack
    n=size(arr)
    do i=1,n
       order(i)=i
    end do
    jstack=0
    l=1
    r=n
    do
       if (r-l < NN) then
          do j=l+1,r
             a=arr(j)
             order_scal=order(j)
             do i=j-1,l,-1
                if (arr(i) <= a) exit
                arr(i+1)=arr(i)
                order(i+1)=order(i)
             end do
             arr(i+1)=a
             order(i+1)=order_scal
          end do
          if (jstack == 0) RETURN
          r=istack(jstack)
          l=istack(jstack-1)
          jstack=jstack-2
       else
          k=(l+r)/2
          call swap(arr(k),arr(l+1))
          call swap(order(k),order(l+1))

          if(arr(l)>arr(r)) then
             call swap(arr(l),arr(r))
             call swap(order(l),order(r))
          end if

          if(arr(l+1)>arr(r)) then
             call swap(arr(l+1),arr(r))
             call swap(order(l+1),order(r))
          end if

          if(arr(l)>arr(l+1)) then
             call swap(arr(l),arr(l+1))
             call swap(order(l),order(l+1))
          end if

          i=l+1
          j=r
          a=arr(l+1)
          order_scal=order(l+1)
          do
             do
                i=i+1
                if (arr(i) >= a) exit
             end do
             do
                j=j-1
                if (arr(j) <= a) exit
             end do
             if (j < i) exit
             call swap(arr(i),arr(j))
             call swap(order(i),order(j))
          end do
          arr(l+1)=arr(j)
          arr(j)=a
          order(l+1)=order(j)
          order(j)=order_scal

          jstack=jstack+2
          if (jstack > NSTACK) pause 'sort: NSTACK too small'
          if (r-i+1 >= j-l) then
             istack(jstack)=r
             istack(jstack-1)=i
             r=j-1
          else
             istack(jstack)=j-1
             istack(jstack-1)=l
             l=i
          end if
       end if
    end do
  END SUBROUTINE sort_i

  SUBROUTINE sort_indx(arr,index)
    IMPLICIT NONE
    REAL(8), DIMENSION(:), INTENT(IN) :: arr
    INTEGER(4), DIMENSION(:), INTENT(OUT) :: index
    INTEGER(4), PARAMETER :: NN=15, NSTACK=50
    REAL(8) :: a
    INTEGER(4) :: n,k,i,j,indext,jstack,l,r
    INTEGER(4), DIMENSION(NSTACK) :: istack
    n=size(arr)
    index=arth_i(1,1,n)
    jstack=0
    l=1
    r=n
    do
       if (r-l < NN) then
          do j=l+1,r
             indext=index(j)
             a=arr(indext)
             do i=j-1,l,-1
                if (arr(index(i)) <= a) exit
                index(i+1)=index(i)
             end do
             index(i+1)=indext
          end do
          if (jstack == 0) RETURN
          r=istack(jstack)
          l=istack(jstack-1)
          jstack=jstack-2
       else
          k=(l+r)/2
          call swap(index(k),index(l+1))
          call icomp_xchg(index(l),index(r))
          call icomp_xchg(index(l+1),index(r))
          call icomp_xchg(index(l),index(l+1))
          i=l+1
          j=r
          indext=index(l+1)
          a=arr(indext)
          do
             do
                i=i+1
                if (arr(index(i)) >= a) exit
             end do
             do
                j=j-1
                if (arr(index(j)) <= a) exit
             end do
             if (j < i) exit
             call swap(index(i),index(j))
          end do
          index(l+1)=index(j)
          index(j)=indext
          jstack=jstack+2
          if (jstack > NSTACK) stop'index: NSTACK too small'
          if (r-i+1 >= j-l) then
             istack(jstack)=r
             istack(jstack-1)=i
             r=j-1
          else
             istack(jstack)=j-1
             istack(jstack-1)=l
             l=i
          end if
       end if
    end do
  CONTAINS
    SUBROUTINE icomp_xchg(i,j)
      INTEGER(4), INTENT(INOUT) :: i,j
      INTEGER(4) :: swp
      if (arr(j) < arr(i)) then
         swp=i
         i=j
         j=swp
      end if
    END SUBROUTINE icomp_xchg

    FUNCTION arth_i(first,increment,n)
      INTEGER(4), INTENT(IN) :: first,increment,n
      INTEGER(4), DIMENSION(n) :: arth_i
      INTEGER(4) :: k,k2,temp
      INTEGER(4), PARAMETER :: NPAR_ARTH=16,NPAR2_ARTH=8
      if (n > 0) arth_i(1)=first
      if (n <= NPAR_ARTH) then
         do k=2,n
            arth_i(k)=arth_i(k-1)+increment
         end do
      else
         do k=2,NPAR2_ARTH
            arth_i(k)=arth_i(k-1)+increment
         end do
         temp=increment*NPAR2_ARTH
         k=NPAR2_ARTH
         do
            if (k >= n) exit
            k2=k+k
            arth_i(k+1:min(k2,n))=temp+arth_i(1:min(k,n-k))
            temp=temp+temp
            k=k2
         end do
      end if
    END FUNCTION arth_i
  END SUBROUTINE sort_indx

  SUBROUTINE swap_i(a,b)
    INTEGER, INTENT(INOUT) :: a,b
    INTEGER :: dum
    dum=a
    a=b
    b=dum
  END SUBROUTINE swap_i
  SUBROUTINE masked_swap_is(a,b,mask)
    INTEGER, INTENT(INOUT) :: a,b
    LOGICAL, INTENT(IN) :: mask
    INTEGER :: swp
    if (mask) then
       swp=a
       a=b
       b=swp
    end if
  END SUBROUTINE masked_swap_is
  
  SUBROUTINE swap_d(a,b)
    REAL(8), INTENT(INOUT) :: a,b
    REAL(8) :: dum
    dum=a
    a=b
    b=dum
  END SUBROUTINE swap_d
  SUBROUTINE masked_swap_ds(a,b,mask)
    REAL(8), INTENT(INOUT) :: a,b
    LOGICAL, INTENT(IN) :: mask
    REAL(8) :: swp
    if (mask) then
       swp=a
       a=b
       b=swp
    end if
  END SUBROUTINE masked_swap_ds
end module moments
!===============================================================================================================================
!===============================================================================================================================


!===============================================================================================================================
!===============================================================================================================================
module kernel !Provides module for non-parametric density and regression (Gaussian). Also contains Mersenne-Twister rng.
  implicit none
  private
  public krnlest
  real(8), parameter :: pi=3.1415926535897932384626433832795
  integer, parameter :: numboot=200
  integer(4), parameter :: &
       &N	=624,&
       &N1	=N+1,&
       &M	=397,&
       &MATA	=-1727483681,&
       &UMASK	=-2147483648,&
       &LMASK	= 2147483647,&
       &TMASKB	=-1658038656,&
       &TMASKC	=-272236544

  integer(4) :: mt(0:N-1),mti,mag01(0:1)
  data mti/N1/
  data mag01/0, MATA/

contains

  subroutine krnlest(inp,outp,bndwdth,gridflag,inplb,inpub,varcalc)
    implicit none
    real(8), dimension(:,:), intent(in) :: inp
    real(8), dimension(:,:), intent(inout) :: outp
    real(8), intent(in) :: bndwdth
    integer, intent(in) :: gridflag
    real(8), optional, intent(in) :: inplb,inpub
    real(8), optional, dimension(:), intent(inout) :: varcalc
    real(8) :: lb,ub
    real(8), allocatable :: bootmat(:,:), newinp(:,:),rnd(:)
    integer :: i,j,boot,regtype,bootflag,xinpdim,dimout
    integer, allocatable :: indx(:),sortvect(:)
    if(present(inplb)) then
       lb=inplb
    else
       lb=minval(inp(:,1))
    end if
    if(present(inpub)) then
       ub=inpub
    else
       ub=maxval(inp(:,1))
    end if
    regtype=size(inp,dim=2)
    bootflag=size(outp,dim=2)/2-1
    dimout=size(outp,dim=1)
    xinpdim=size(inp,dim=1)

    if(gridflag .eq. 0) then
       outp=dble(0)
       do i=1,dimout
          outp(i,1)=lb+(i-1)*(ub-lb)/(dimout-1)
       end do
    end if

    if (regtype .eq. 1) then
       call krnlest_pdf(inp(:,1),outp(:,1),outp(:,2))
    else
       call krnlest_reg(inp,outp(:,1),outp(:,2))
    end if

    if(bootflag .eq. 1) then
       call mtseed(int(secnds(0.0)*1000.0)) !Initialize Mersenne-Twister by system clock
       do i=1,10000 !Warm up MT
          j=mtgen()
       end do
       allocate(bootmat(dimout,numboot),newinp(xinpdim,size(inp,dim=2)),rnd(xinpdim),indx(xinpdim),sortvect(numboot))
       bootmat=dble(0)
       do boot=1,numboot
          do i=1,xinpdim
             newinp(i,:)=inp(nint(mtuni()*dble(xinpdim-1))+1,:)
          end do
          if (regtype .eq. 1) then
             call krnlest_pdf(newinp(:,1),outp(:,1),bootmat(:,boot))
          else
             if(present(varcalc)) then
                call krnlest_reg_novar(newinp,outp(:,1),varcalc)
                call krnlest_reg(newinp,outp(:,1),bootmat(:,boot))
             else
                call krnlest_reg(newinp,outp(:,1),bootmat(:,boot))
             end if
          end if
       end do
       do i=1,dimout
          forall (j=1:numboot) sortvect(j)=j
          call sort_d(bootmat(i,:),sortvect)
          outp(i,3)=bootmat(i,sortvect(nint(.05*real(numboot))))
          outp(i,4)=bootmat(i,sortvect(nint(.95*real(numboot))))
       end do
    end if
  contains
    subroutine krnlest_pdf(inp1,inp2,outp1)
      implicit none
      real(8), dimension(:), intent(in) :: inp1
      real(8), dimension(:), intent(in) :: inp2
      real(8), dimension(:), intent(out) :: outp1
      do i=1,dimout
         do j=1,xinpdim
            outp1(i)=outp1(i)+exp(-dble(.5)*((inp2(i)-inp1(j))/bndwdth)*((inp2(i)-inp1(j))/bndwdth))
         end do
         outp1(i)=outp1(i)/((sqrt(dble(2)*pi))*bndwdth*xinpdim)
      end do
    end subroutine krnlest_pdf

    subroutine krnlest_reg(inp1,inp2,outp1)
      implicit none
      real(8), dimension(:,:), intent(in) :: inp1
      real(8), dimension(:), intent(in) :: inp2
      real(8), dimension(:), intent(out) :: outp1
      real(8) :: wghtsum,x
      do i=1,dimout
         wghtsum=dble(0)
         do j=1,xinpdim
            x=exp(-dble(.5)*((inp2(i)-inp1(j,1))/bndwdth)*((inp2(i)-inp1(j,1))/bndwdth))
            wghtsum=wghtsum+x
            if(present(varcalc)) then
               outp1(i)=outp1(i)+inp1(j,2)*inp1(j,2)*x
            else
               outp1(i)=outp1(i)+inp1(j,2)*x
            end if
         end do
         if(present(varcalc)) then
            outp1(i)=max(dble(0),outp1(i)/wghtsum-varcalc(i)*varcalc(i))
         else
            outp1(i)=outp1(i)/wghtsum
         end if
      end do
    end subroutine krnlest_reg

    subroutine krnlest_reg_novar(inp1,inp2,outp1)
      implicit none
      real(8), dimension(:,:), intent(in) :: inp1
      real(8), dimension(:), intent(in) :: inp2
      real(8), dimension(:), intent(out) :: outp1
      real(8) :: wghtsum,x
      do i=1,dimout
         wghtsum=dble(0)
         do j=1,xinpdim
            x=exp(-dble(.5)*((inp2(i)-inp1(j,1))/bndwdth)*((inp2(i)-inp1(j,1))/bndwdth))
            wghtsum=wghtsum+x
            outp1(i)=outp1(i)+inp1(j,2)*x
         end do
         outp1(i)=outp1(i)/wghtsum
      end do
    end subroutine krnlest_reg_novar
  end subroutine krnlest

  SUBROUTINE Sort_d(Xin,ind)
    !  Sorts xout into ascENDing order - Quicksort
    real(8), DIMENSION (:), INTENT (in) :: Xin
    INTEGER, INTENT(INOUT) :: ind(:)
    real(8) :: xout(size(ind))
    integer :: dim
    dim=size(ind)
    xout=Xin(ind)
    Call SUBSOR (xout,1, dim,ind)
    Call INSSORT (xout,ind)
  CONTAINS
    RECURSIVE SUBROUTINE SUBSOR (xout,IDEB1, IFIN1,ind)
      !  Sorts xout from IDEB1 to IFIN1
      real(8), INTENT (InOut) :: xout(dim)
      INTEGER, INTENT (In) :: IDEB1, IFIN1
      INTEGER, INTENT(INOUT) :: ind(dim)
      INTEGER, PARAMETER :: NINS = 16 ! Max for insertion sort
      INTEGER :: ICRS, IDEB, IDCR, IFIN, IMIL,IWRK,IPIV
      REAL(8) :: XPIV, XWRK
      IDEB = IDEB1
      IFIN = IFIN1
      !  IF we DOn't have enough values to make it worth while, we leave
      !  them unsorted, and the final insertion sort will take care of them
      IF ((IFIN - IDEB) > NINS) THEN
         IMIL = (IDEB+IFIN) / 2
         !  One chooses a pivot, median of 1st, last, and middle values
         IF (xout(IMIL) < xout(IDEB)) THEN
            XWRK = xout (IDEB)
            IWRK = IND (IDEB)
            xout (IDEB) = xout (IMIL)
            IND (IDEB) = IND (IMIL)
            xout (IMIL) = XWRK
            IND (IMIL) = IWRK
         END IF
         IF (xout(IMIL) > xout(IFIN)) Then
            XWRK = xout (IFIN)
            IWRK = IND(IFIN)
            xout (IFIN) = xout (IMIL)
            IND (IFIN) = IND (IMIL)
            xout (IMIL) = XWRK
            IND (IMIL) = IWRK
            IF (xout(IMIL) < xout(IDEB)) Then
               XWRK = xout (IDEB)
               IWRK = IND (IDEB)
               xout (IDEB) = xout (IMIL)
               IND (IDEB) = IND (IMIL)
               xout (IMIL) = XWRK
               IND (IMIL) = IWRK
            END IF
         END IF
         XPIV = xout (IMIL)
         IPIV = IND (IMIL)
         !  One exchanges values to put those > pivot in the END and
         !  those <= pivot at the beginning
         ICRS = IDEB
         IDCR = IFIN
         ECH2: DO
            DO
               ICRS = ICRS + 1
               IF (ICRS >= IDCR) Then
                  !  the first  >  pivot is IDCR
                  !  the last   <= pivot is ICRS-1
                  !  Note: IF one arrives here on the first iteration, then
                  !  the pivot is the maximum of the set, the last value is equal
                  !  to it, and one can reduce by one the size of the set to process,
                  !  as IF xout (IFIN) > XPIV
                  Exit ECH2
               END IF
               IF (xout(ICRS) > XPIV) Exit
            END DO
            DO
               IF (xout(IDCR) <= XPIV) Exit
               IDCR = IDCR - 1
               IF (ICRS >= IDCR) Then
                  !  The last value < pivot is always ICRS-1
                  Exit ECH2
               END IF
            END DO
            XWRK = xout (IDCR)
            IWRK = IND (IDCR)
            xout (IDCR) = xout (ICRS)
            IND (IDCR) = IND (ICRS)
            xout (ICRS) = XWRK
            IND (ICRS) = IWRK
         END DO ECH2
         !  One now sorts each of the two sub-intervals
         Call SUBSOR (xout,IDEB1, ICRS-1,ind)
         Call SUBSOR (xout,IDCR, IFIN1,ind)
      END IF
      RETURN
    END SUBROUTINE SUBSOR
    
    SUBROUTINE INSSORT (xout,IND)
      !  Sorts xout into increasing order (Insertion sort)
      real(8), INTENT (InOut) :: xout(dim)
      INTEGER, INTENT(INOUT) :: ind(dim)
      INTEGER :: ICRS, IDCR,IWRK
      REAL(8) :: XWRK
      DO ICRS = 2, dim
         XWRK = xout (ICRS)
         IWRK = IND (ICRS)
         IF (XWRK >= xout(ICRS-1)) Cycle
         xout (ICRS) = xout (ICRS-1)
         IND (ICRS) = IND (ICRS-1)
         DO IDCR = ICRS - 2, 1, - 1
            IF (XWRK >= xout(IDCR)) Exit
            xout (IDCR+1) = xout (IDCR)
            IND (IDCR+1) = IND (IDCR)
         END DO
         xout (IDCR+1) = XWRK
         IND (IDCR+1) = IWRK
      END DO
      RETURN
    END SUBROUTINE INSSORT
  END SUBROUTINE Sort_d

  function mtuni()
    implicit none
    real(8) :: mtuni
    mtuni=(dble(mtgen())+2147483649.0d0)/4294967297.0d0 !Produces 2^32 possible realizations in the (0,1) interval (not including 0 and 1).
  end function mtuni

  function mtgen()
    implicit none
    integer(4) :: y,mtgen,TSHFTU,TSHFTS,TSHFTT,TSHFTL,kk
    TSHFTU(y)=ishft(y,-11)
    TSHFTS(y)=ishft(y,7)
    TSHFTT(y)=ishft(y,15)
    TSHFTL(y)=ishft(y,-18)

    if(mti.ge.N) then
       if(mti.eq.N+1) call mtseed(19650218)
       do kk=0,N-M-1
          y=ior(iand(mt(kk),UMASK),iand(mt(kk+1),LMASK))
          mt(kk)=ieor(ieor(mt(kk+M),ishft(y,-1)),mag01(iand(y,1)))
       end do
       do kk=N-M,N-2
          y=ior(iand(mt(kk),UMASK),iand(mt(kk+1),LMASK))
          mt(kk)=ieor(ieor(mt(kk+(M-N)),ishft(y,-1)),mag01(iand(y,1)))
       end do
       y=ior(iand(mt(N-1),UMASK),iand(mt(0),LMASK))
       mt(N-1)=ieor(ieor(mt(M-1),ishft(y,-1)),mag01(iand(y,1)))
       mti = 0
    endif

    y=mt(mti)
    mti=mti+1
    y=ieor(y,TSHFTU(y))
    y=ieor(y,iand(TSHFTS(y),TMASKB))
    y=ieor(y,iand(TSHFTT(y),TMASKC))
    y=ieor(y,TSHFTL(y))
    mtgen=y
  end function mtgen

  subroutine mtseed(seed)
    implicit none
    integer(4), intent(in) :: seed
    mt(0)= seed
    do mti=1,N-1
       mt(mti)=iand(4294967295,1812433253*ieor(mt(mti-1),ishft(mt(mti-1),-30)) + mti)
    end do
  end subroutine mtseed
end module kernel
!===============================================================================================================================
!===============================================================================================================================

!===============================================================================================================================
!===============================================================================================================================
module distr !Makes Pareto and Weibull distributions available (numerical integration in gamma fct done by IMSL routine).
  use Numerical_Libraries
  implicit none
  private
  interface weibull_pdf
     module procedure weibullpdf_dp
  end interface
  interface weibull_cdf
     module procedure weibullcdf_dp
  end interface
  interface weibull_invcdf
     module procedure weibullinvcdf_dp
  end interface
  interface pareto_pdf
     module procedure pareto_pdf_dp
  end interface
  interface pareto_cdf
     module procedure pareto_cdf_dp
  end interface
  real(8), parameter :: one=1.0000000000000000d0, zero=0.0000000000000000d0
  public weibull_pdf,weibull_cdf,weibull_invcdf,pareto_pdf,pareto_cdf,gammafct
contains

  double precision function weibullpdf_dp(w_dur,w_beta,w_eta) !two-parameter weibull (origin set at zero. User can shift distribution post call).
    implicit none
    real(8), intent(in) :: w_dur,w_beta,w_eta
    real(8) :: w_loc
    if(w_dur .lt. dble(0)) pause 'Stop (weibullpdf_dp): w_dur must be at least 0'
    w_loc=w_dur/w_eta
    weibullpdf_dp=(w_beta/w_eta)*(w_loc**(w_beta-one))*exp(-w_loc**w_beta)
  end function weibullpdf_dp

  double precision function weibullcdf_dp(w_dur,w_beta,w_eta)
    implicit none
    real(8), intent(in) :: w_dur,w_beta,w_eta
    real(8) :: w_loc
    if(w_dur .lt. dble(0)) pause 'Stop (weibullcdf_dp): w_dur must be at least 0'
    if(w_eta .eq. dble(0)) then
       weibullcdf_dp=one
    else
       w_loc=w_dur/w_eta
       weibullcdf_dp=one-exp(-w_loc**w_beta)
    end if
  end function weibullcdf_dp

  double precision function weibullinvcdf_dp(w_cdf,w_beta,w_eta)
    implicit none
    real(8), intent(in) :: w_cdf,w_beta,w_eta
    if(w_cdf .lt. dble(0) .or. w_cdf .gt. dble(1)) pause 'Stop (weibullinvcdf_dp): w_cdf must be at least 0 and no greater than 1'
    weibullinvcdf_dp=w_eta*((log(one/(one-w_cdf))**(one/w_beta)))
  end function weibullinvcdf_dp

  double precision function gammafct(glob_inp)
    implicit none
    real(8),intent(in) :: glob_inp
    real(8) :: loc_huge,errest
    interface
       subroutine dqdagi(f, bound, interv, errabs, errrel, result, errest)
         integer	interv
         double precision f, bound, errabs, errrel, result, errest
         external   f
       end subroutine dqdagi
    end interface
    loc_huge=log(huge(loc_huge))
    call dqdagi(gamma_integrant,dble(0),1,dble(1.0e-10),dble(1.0e-10),gammafct,errest)
  contains
    double precision function gamma_integrant(loc_inp)
      implicit none
      real(8), intent(in) :: loc_inp
      if(loc_inp .gt. loc_huge) then
         gamma_integrant=dble(0)
      else
         gamma_integrant=exp(-loc_inp)*(loc_inp**(glob_inp-dble(1)))
      end if
    end function gamma_integrant
  end function gammafct

  function pareto_pdf_dp(eval_point,shape,scale) !Pareto PDF
    implicit none
    real(8), intent(in) :: eval_point,shape,scale
    real(8) :: pareto_pdf_dp
    if(shape .le. zero) stop'Fatal (pareto_pdf_dp - distr module): shape parameter must be greater than zero'
    if(scale .le. zero) stop'Fatal (pareto_pdf_dp - distr module): scale parameter must be greater than zero'
    if(eval_point .lt. scale) then
       pareto_pdf_dp=zero
    else
       pareto_pdf_dp=shape*(scale**shape)/(eval_point**(shape+one))
    end if
  end function pareto_pdf_dp

  function pareto_cdf_dp(eval_point,shape,scale) !Pareto CDF with zero origin
    implicit none
    real(8), intent(in) :: eval_point,shape,scale
    real(8) :: pareto_cdf_dp
    if(shape .le. zero) stop'Fatal (pareto_cdf_dp - distr module): shape parameter must be greater than zero'
    if(scale .le. zero) stop'Fatal (pareto_cdf_dp - distr module): scale parameter must be greater than zero'
    if(eval_point .lt. scale) then
       pareto_cdf_dp=zero
    else
       pareto_cdf_dp=one-(scale/eval_point)**shape
    end if
  end function pareto_cdf_dp
end module distr
!===============================================================================================================================
!===============================================================================================================================

!===============================================================================================================================
!===============================================================================================================================
module mpivar
  implicit none
  include "/share/apps/mpich2/include/mpif.h"
  integer :: end_stat,ierr,myid,numprocs,status(MPI_STATUS_SIZE)
  logical :: MASTER
  real(8) :: starttime, endtime

contains
  subroutine mpi_ini
    implicit none
    call MPI_INIT(ierr)
    call MPI_COMM_RANK(MPI_COMM_WORLD,myid,ierr)
    call MPI_COMM_SIZE(MPI_COMM_WORLD,numprocs,ierr)
    master=(myid==0)
    print *,'Hello. I am processor ',myid,' out of ',numprocs
    if(master) starttime=MPI_WTIME()
  end subroutine mpi_ini

  subroutine mpi_term
    implicit none
    if(master) then
       endtime=MPI_WTIME()
       print *,'Elapsed time is ', endtime-starttime, ' seconds.'
    end if
    !  print *,'I am processor ',myid, '. I am done.'
    call MPI_FINALIZE(ierr)
  end subroutine mpi_term
end module mpivar
!===============================================================================================================================
!===============================================================================================================================
